# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.resources.types import customer_user_access
from google.protobuf import field_mask_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.services",
    marshal="google.ads.googleads.v20",
    manifest={
        "MutateCustomerUserAccessRequest",
        "CustomerUserAccessOperation",
        "MutateCustomerUserAccessResponse",
        "MutateCustomerUserAccessResult",
    },
)


class MutateCustomerUserAccessRequest(proto.Message):
    r"""Mutate Request for
    [CustomerUserAccessService.MutateCustomerUserAccess][google.ads.googleads.v20.services.CustomerUserAccessService.MutateCustomerUserAccess].

    Attributes:
        customer_id (str):
            Required. The ID of the customer being
            modified.
        operation (google.ads.googleads.v20.services.types.CustomerUserAccessOperation):
            Required. The operation to perform on the
            customer
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operation: "CustomerUserAccessOperation" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="CustomerUserAccessOperation",
    )


class CustomerUserAccessOperation(proto.Message):
    r"""A single operation (update, remove) on customer user access.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        update (google.ads.googleads.v20.resources.types.CustomerUserAccess):
            Update operation: The customer user access is
            expected to have a valid resource name.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: A resource name for the removed access is
            expected, in this format:

            ``customers/{customer_id}/customerUserAccesses/{CustomerUserAccess.user_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=3,
        message=field_mask_pb2.FieldMask,
    )
    update: customer_user_access.CustomerUserAccess = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=customer_user_access.CustomerUserAccess,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=2,
        oneof="operation",
    )


class MutateCustomerUserAccessResponse(proto.Message):
    r"""Response message for customer user access mutate.

    Attributes:
        result (google.ads.googleads.v20.services.types.MutateCustomerUserAccessResult):
            Result for the mutate.
    """

    result: "MutateCustomerUserAccessResult" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="MutateCustomerUserAccessResult",
    )


class MutateCustomerUserAccessResult(proto.Message):
    r"""The result for the customer user access mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.services",
    marshal="google.ads.googleads.v20",
    manifest={
        "GenerateShareablePreviewsRequest",
        "ShareablePreview",
        "AssetGroupIdentifier",
        "GenerateShareablePreviewsResponse",
        "ShareablePreviewOrError",
        "ShareablePreviewResult",
    },
)


class GenerateShareablePreviewsRequest(proto.Message):
    r"""Request message for
    [ShareablePreviewService.GenerateShareablePreviews][google.ads.googleads.v20.services.ShareablePreviewService.GenerateShareablePreviews].

    Attributes:
        customer_id (str):
            Required. The customer creating the shareable
            previews request.
        shareable_previews (MutableSequence[google.ads.googleads.v20.services.types.ShareablePreview]):
            Required. The list of shareable previews to
            generate.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    shareable_previews: MutableSequence["ShareablePreview"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="ShareablePreview",
        )
    )


class ShareablePreview(proto.Message):
    r"""A shareable preview with its identifier.

    Attributes:
        asset_group_identifier (google.ads.googleads.v20.services.types.AssetGroupIdentifier):
            Required. Asset group of the shareable
            preview.
    """

    asset_group_identifier: "AssetGroupIdentifier" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="AssetGroupIdentifier",
    )


class AssetGroupIdentifier(proto.Message):
    r"""Asset group of the shareable preview.

    Attributes:
        asset_group_id (int):
            Required. The asset group identifier.
    """

    asset_group_id: int = proto.Field(
        proto.INT64,
        number=1,
    )


class GenerateShareablePreviewsResponse(proto.Message):
    r"""Response message for
    [ShareablePreviewService.GenerateShareablePreviews][google.ads.googleads.v20.services.ShareablePreviewService.GenerateShareablePreviews].

    Attributes:
        responses (MutableSequence[google.ads.googleads.v20.services.types.ShareablePreviewOrError]):
            List of generate shareable preview results.
    """

    responses: MutableSequence["ShareablePreviewOrError"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="ShareablePreviewOrError",
    )


class ShareablePreviewOrError(proto.Message):
    r"""Result of the generate shareable preview.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        asset_group_identifier (google.ads.googleads.v20.services.types.AssetGroupIdentifier):
            The asset group of the shareable preview.
        shareable_preview_result (google.ads.googleads.v20.services.types.ShareablePreviewResult):
            The shareable preview result.

            This field is a member of `oneof`_ ``generate_shareable_preview_response``.
        partial_failure_error (google.rpc.status_pb2.Status):
            The shareable preview partial failure error.

            This field is a member of `oneof`_ ``generate_shareable_preview_response``.
    """

    asset_group_identifier: "AssetGroupIdentifier" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="AssetGroupIdentifier",
    )
    shareable_preview_result: "ShareablePreviewResult" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="generate_shareable_preview_response",
        message="ShareablePreviewResult",
    )
    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="generate_shareable_preview_response",
        message=status_pb2.Status,
    )


class ShareablePreviewResult(proto.Message):
    r"""Message to hold a shareable preview result.

    Attributes:
        shareable_preview_url (str):
            The shareable preview URL.
        expiration_date_time (str):
            Expiration date time using the ISO-8601
            format.
    """

    shareable_preview_url: str = proto.Field(
        proto.STRING,
        number=1,
    )
    expiration_date_time: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "ManagerLinkErrorEnum",
    },
)


class ManagerLinkErrorEnum(proto.Message):
    r"""Container for enum describing possible ManagerLink errors."""

    class ManagerLinkError(proto.Enum):
        r"""Enum describing possible ManagerLink errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            ACCOUNTS_NOT_COMPATIBLE_FOR_LINKING (2):
                The manager and client have incompatible
                account types.
            TOO_MANY_MANAGERS (3):
                Client is already linked to too many
                managers.
            TOO_MANY_INVITES (4):
                Manager has too many pending invitations.
            ALREADY_INVITED_BY_THIS_MANAGER (5):
                Client is already invited by this manager.
            ALREADY_MANAGED_BY_THIS_MANAGER (6):
                The client is already managed by this
                manager.
            ALREADY_MANAGED_IN_HIERARCHY (7):
                Client is already managed in hierarchy.
            DUPLICATE_CHILD_FOUND (8):
                Manager and sub-manager to be linked have
                duplicate client.
            CLIENT_HAS_NO_ADMIN_USER (9):
                Client has no active user that can access the
                client account.
            MAX_DEPTH_EXCEEDED (10):
                Adding this link would exceed the maximum
                hierarchy depth.
            CYCLE_NOT_ALLOWED (11):
                Adding this link will create a cycle.
            TOO_MANY_ACCOUNTS (12):
                Manager account has the maximum number of
                linked clients.
            TOO_MANY_ACCOUNTS_AT_MANAGER (13):
                Parent manager account has the maximum number
                of linked clients.
            NON_OWNER_USER_CANNOT_MODIFY_LINK (14):
                The account is not authorized owner.
            SUSPENDED_ACCOUNT_CANNOT_ADD_CLIENTS (15):
                Your manager account is suspended, and you
                are no longer allowed to link to clients.
            CLIENT_OUTSIDE_TREE (16):
                You are not allowed to move a client to a
                manager that is not under your current
                hierarchy.
            INVALID_STATUS_CHANGE (17):
                The changed status for mutate link is
                invalid.
            INVALID_CHANGE (18):
                The change for mutate link is invalid.
            CUSTOMER_CANNOT_MANAGE_SELF (19):
                You are not allowed to link a manager account
                to itself.
            CREATING_ENABLED_LINK_NOT_ALLOWED (20):
                The link was created with status ACTIVE and
                not PENDING.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ACCOUNTS_NOT_COMPATIBLE_FOR_LINKING = 2
        TOO_MANY_MANAGERS = 3
        TOO_MANY_INVITES = 4
        ALREADY_INVITED_BY_THIS_MANAGER = 5
        ALREADY_MANAGED_BY_THIS_MANAGER = 6
        ALREADY_MANAGED_IN_HIERARCHY = 7
        DUPLICATE_CHILD_FOUND = 8
        CLIENT_HAS_NO_ADMIN_USER = 9
        MAX_DEPTH_EXCEEDED = 10
        CYCLE_NOT_ALLOWED = 11
        TOO_MANY_ACCOUNTS = 12
        TOO_MANY_ACCOUNTS_AT_MANAGER = 13
        NON_OWNER_USER_CANNOT_MODIFY_LINK = 14
        SUSPENDED_ACCOUNT_CANNOT_ADD_CLIENTS = 15
        CLIENT_OUTSIDE_TREE = 16
        INVALID_STATUS_CHANGE = 17
        INVALID_CHANGE = 18
        CUSTOMER_CANNOT_MANAGE_SELF = 19
        CREATING_ENABLED_LINK_NOT_ALLOWED = 20


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "CustomAudienceTypeEnum",
    },
)


class CustomAudienceTypeEnum(proto.Message):
    r"""The types of custom audience."""

    class CustomAudienceType(proto.Enum):
        r"""Enum containing possible custom audience types.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            AUTO (2):
                Google Ads will auto-select the best
                interpretation at serving time.
            INTEREST (3):
                Matches users by their interests.
            PURCHASE_INTENT (4):
                Matches users by topics they are researching
                or products they are considering for purchase.
            SEARCH (5):
                Matches users by what they searched on Google
                Search.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        AUTO = 2
        INTEREST = 3
        PURCHASE_INTENT = 4
        SEARCH = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.common.types import goal_setting
from google.ads.googleads.v22.enums.types import goal_optimization_eligibility
from google.ads.googleads.v22.enums.types import goal_type as gage_goal_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "Goal",
    },
)


class Goal(proto.Message):
    r"""Representation of goals.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the goal. Goal resource
            names have the form:
            ``customers/{customer_id}/goals/{goal_id}``
        goal_id (int):
            Output only. The ID of this goal.

            This field is a member of `oneof`_ ``_goal_id``.
        goal_type (google.ads.googleads.v22.enums.types.GoalTypeEnum.GoalType):
            Output only. The type of this goal.
        owner_customer (str):
            Output only. The resource name of the goal
            owner customer.

            This field is a member of `oneof`_ ``_owner_customer``.
        optimization_eligibility (google.ads.googleads.v22.enums.types.GoalOptimizationEligibilityEnum.GoalOptimizationEligibility):
            Output only. Indicates if this goal is
            eligible for campaign optimization.
        retention_goal_settings (google.ads.googleads.v22.common.types.GoalSetting.RetentionGoal):
            Retention goal settings.

            This field is a member of `oneof`_ ``goal_settings``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    goal_id: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )
    goal_type: gage_goal_type.GoalTypeEnum.GoalType = proto.Field(
        proto.ENUM,
        number=3,
        enum=gage_goal_type.GoalTypeEnum.GoalType,
    )
    owner_customer: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    optimization_eligibility: (
        goal_optimization_eligibility.GoalOptimizationEligibilityEnum.GoalOptimizationEligibility
    ) = proto.Field(
        proto.ENUM,
        number=6,
        enum=goal_optimization_eligibility.GoalOptimizationEligibilityEnum.GoalOptimizationEligibility,
    )
    retention_goal_settings: goal_setting.GoalSetting.RetentionGoal = (
        proto.Field(
            proto.MESSAGE,
            number=7,
            oneof="goal_settings",
            message=goal_setting.GoalSetting.RetentionGoal,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.common.types import asset_policy
from google.ads.googleads.v21.enums.types import asset_field_type
from google.ads.googleads.v21.enums.types import asset_link_primary_status
from google.ads.googleads.v21.enums.types import (
    asset_link_primary_status_reason,
)
from google.ads.googleads.v21.enums.types import asset_link_status
from google.ads.googleads.v21.enums.types import asset_source


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "AdGroupAsset",
    },
)


class AdGroupAsset(proto.Message):
    r"""A link between an ad group and an asset.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the ad group asset.
            AdGroupAsset resource names have the form:

            ``customers/{customer_id}/adGroupAssets/{ad_group_id}~{asset_id}~{field_type}``
        ad_group (str):
            Required. Immutable. The ad group to which
            the asset is linked.
        asset (str):
            Required. Immutable. The asset which is
            linked to the ad group.
        field_type (google.ads.googleads.v21.enums.types.AssetFieldTypeEnum.AssetFieldType):
            Required. Immutable. Role that the asset
            takes under the linked ad group.
        source (google.ads.googleads.v21.enums.types.AssetSourceEnum.AssetSource):
            Output only. Source of the adgroup asset
            link.
        status (google.ads.googleads.v21.enums.types.AssetLinkStatusEnum.AssetLinkStatus):
            Status of the ad group asset.
        primary_status (google.ads.googleads.v21.enums.types.AssetLinkPrimaryStatusEnum.AssetLinkPrimaryStatus):
            Output only. Provides the PrimaryStatus of
            this asset link. Primary status is meant
            essentially to differentiate between the plain
            "status" field, which has advertiser set values
            of enabled, paused, or removed.  The primary
            status takes into account other signals (for
            assets its mainly policy and quality approvals)
            to come up with a more comprehensive status to
            indicate its serving state.
        primary_status_details (MutableSequence[google.ads.googleads.v21.common.types.AssetLinkPrimaryStatusDetails]):
            Output only. Provides the details of the
            primary status and its associated reasons.
        primary_status_reasons (MutableSequence[google.ads.googleads.v21.enums.types.AssetLinkPrimaryStatusReasonEnum.AssetLinkPrimaryStatusReason]):
            Output only. Provides a list of reasons for
            why an asset is not serving or not serving at
            full capacity.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    ad_group: str = proto.Field(
        proto.STRING,
        number=2,
    )
    asset: str = proto.Field(
        proto.STRING,
        number=3,
    )
    field_type: asset_field_type.AssetFieldTypeEnum.AssetFieldType = (
        proto.Field(
            proto.ENUM,
            number=4,
            enum=asset_field_type.AssetFieldTypeEnum.AssetFieldType,
        )
    )
    source: asset_source.AssetSourceEnum.AssetSource = proto.Field(
        proto.ENUM,
        number=6,
        enum=asset_source.AssetSourceEnum.AssetSource,
    )
    status: asset_link_status.AssetLinkStatusEnum.AssetLinkStatus = proto.Field(
        proto.ENUM,
        number=5,
        enum=asset_link_status.AssetLinkStatusEnum.AssetLinkStatus,
    )
    primary_status: (
        asset_link_primary_status.AssetLinkPrimaryStatusEnum.AssetLinkPrimaryStatus
    ) = proto.Field(
        proto.ENUM,
        number=7,
        enum=asset_link_primary_status.AssetLinkPrimaryStatusEnum.AssetLinkPrimaryStatus,
    )
    primary_status_details: MutableSequence[
        asset_policy.AssetLinkPrimaryStatusDetails
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=8,
        message=asset_policy.AssetLinkPrimaryStatusDetails,
    )
    primary_status_reasons: MutableSequence[
        asset_link_primary_status_reason.AssetLinkPrimaryStatusReasonEnum.AssetLinkPrimaryStatusReason
    ] = proto.RepeatedField(
        proto.ENUM,
        number=9,
        enum=asset_link_primary_status_reason.AssetLinkPrimaryStatusReasonEnum.AssetLinkPrimaryStatusReason,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import (
    data_link_status as gage_data_link_status,
)
from google.ads.googleads.v21.resources.types import data_link as gagr_data_link


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "CreateDataLinkRequest",
        "CreateDataLinkResponse",
        "RemoveDataLinkRequest",
        "RemoveDataLinkResponse",
        "UpdateDataLinkRequest",
        "UpdateDataLinkResponse",
    },
)


class CreateDataLinkRequest(proto.Message):
    r"""Request message for
    [DataLinkService.CreateDataLink][google.ads.googleads.v21.services.DataLinkService.CreateDataLink].

    Attributes:
        customer_id (str):
            Required. The ID of the customer for which
            the data link is created.
        data_link (google.ads.googleads.v21.resources.types.DataLink):
            Required. The data link to be created.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    data_link: gagr_data_link.DataLink = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gagr_data_link.DataLink,
    )


class CreateDataLinkResponse(proto.Message):
    r"""Response message for
    [DataLinkService.CreateDataLink][google.ads.googleads.v21.services.DataLinkService.CreateDataLink].

    Attributes:
        resource_name (str):
            Returned for successful operations. Resource
            name of the data link.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class RemoveDataLinkRequest(proto.Message):
    r"""Request message for
    [DataLinkService.RemoveDataLink][google.ads.googleads.v21.services.DataLinkService.RemoveDataLink].

    Attributes:
        customer_id (str):
            Required. The ID of the customer for which
            the data link is updated.
        resource_name (str):
            Required. The data link is expected to have a
            valid resource name.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    resource_name: str = proto.Field(
        proto.STRING,
        number=2,
    )


class RemoveDataLinkResponse(proto.Message):
    r"""Response message for
    [DataLinkService.RemoveDataLink][google.ads.googleads.v21.services.DataLinkService.RemoveDataLink].

    Attributes:
        resource_name (str):
            Result for the remove request.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class UpdateDataLinkRequest(proto.Message):
    r"""Request message for
    [DataLinkService.UpdateDataLink][google.ads.googleads.v21.services.DataLinkService.UpdateDataLink].

    Attributes:
        customer_id (str):
            Required. The ID of the customer for which
            the data link is created.
        data_link_status (google.ads.googleads.v21.enums.types.DataLinkStatusEnum.DataLinkStatus):
            Required. The data link status to be updated
            to.
        resource_name (str):
            Required. The data link is expected to have a
            valid resource name.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    data_link_status: (
        gage_data_link_status.DataLinkStatusEnum.DataLinkStatus
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=gage_data_link_status.DataLinkStatusEnum.DataLinkStatus,
    )
    resource_name: str = proto.Field(
        proto.STRING,
        number=3,
    )


class UpdateDataLinkResponse(proto.Message):
    r"""Response message for
    [DataLinkService.UpdateDataLink][google.ads.googleads.v21.services.DataLinkService.UpdateDataLink].

    Attributes:
        resource_name (str):
            Returned for successful operations. Resource
            name of the data link.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example shows how to create a Performance Max retail campaign.

This will be created for "All products".

For more information about Performance Max retail campaigns, see
https://developers.google.com/google-ads/api/docs/performance-max/retail

Prerequisites:
- You need to have access to a Merchant Center account. You can find
  instructions to create a Merchant Center account here:
  https://support.google.com/merchants/answer/188924.
  This account must be linked to your Google Ads account. The integration
  instructions can be found at:
  https://developers.google.com/google-ads/api/docs/shopping-ads/merchant-center
- You need your Google Ads account to track conversions. The different ways
  to track conversions can be found here:
  https://support.google.com/google-ads/answer/1722054.
- You must have at least one conversion action in the account. For
  more about conversion actions, see
  https://developers.google.com/google-ads/api/docs/conversions/overview#conversion_actions
"""


import argparse
from datetime import datetime, timedelta
import sys
from typing import Dict, List, Union
from uuid import uuid4

from google.api_core import protobuf_helpers

from examples.utils.example_helpers import get_image_bytes_from_url
from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.util import convert_snake_case_to_upper_case
from google.ads.googleads.v22.enums.types.conversion_action_category import (
    ConversionActionCategoryEnum,
)
from google.ads.googleads.v22.enums.types.conversion_origin import (
    ConversionOriginEnum,
)
from google.ads.googleads.v22.enums.types.asset_field_type import (
    AssetFieldTypeEnum,
)
from google.ads.googleads.v22.resources.types.asset import Asset
from google.ads.googleads.v22.resources.types.asset_group import AssetGroup
from google.ads.googleads.v22.resources.types.asset_group_asset import (
    AssetGroupAsset,
)
from google.ads.googleads.v22.resources.types.asset_group_listing_group_filter import (
    AssetGroupListingGroupFilter,
)
from google.ads.googleads.v22.resources.types.campaign import Campaign
from google.ads.googleads.v22.resources.types.campaign_asset import (
    CampaignAsset,
)
from google.ads.googleads.v22.resources.types.campaign_budget import (
    CampaignBudget,
)
from google.ads.googleads.v22.resources.types.campaign_conversion_goal import (
    CampaignConversionGoal,
)
from google.ads.googleads.v22.resources.types.campaign_criterion import (
    CampaignCriterion,
)
from google.ads.googleads.v22.services.services.asset_group_service import (
    AssetGroupServiceClient,
)
from google.ads.googleads.v22.services.services.asset_service import (
    AssetServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_budget_service import (
    CampaignBudgetServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_conversion_goal_service import (
    CampaignConversionGoalServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_service import (
    CampaignServiceClient,
)
from google.ads.googleads.v22.services.services.geo_target_constant_service import (
    GeoTargetConstantServiceClient,
)
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    MutateGoogleAdsResponse,
    MutateOperationResponse,
    SearchGoogleAdsRequest,
    SearchGoogleAdsResponse,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    MutateOperation,
)

# We specify temporary IDs that are specific to a single mutate request.
# Temporary IDs are always negative and unique within one mutate request.
#
# See https://developers.google.com/google-ads/api/docs/mutating/best-practices
# for further details.
#
# These temporary IDs are fixed because they are used in multiple places.
_BUDGET_TEMPORARY_ID: str = "-1"
_PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID: str = "-2"
_ASSET_GROUP_TEMPORARY_ID: str = "-3"

# There are also entities that will be created in the same request but do not
# need to be fixed temporary IDs because they are referenced only once.
_next_temp_id: int = int(_ASSET_GROUP_TEMPORARY_ID) - 1


# [START add_performance_max_retail_campaign]
def main(
    client: GoogleAdsClient,
    customer_id: str,
    merchant_center_account_id: int,
    final_url: str,
    brand_guidelines_enabled: bool,
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        merchant_center_account_id: The Merchant Center account ID.
        final_url: the final URL.
        brand_guidelines_enabled: a boolean value indicating if the campaign is
          enabled for brand guidelines.
    """
    # [START add_performance_max_retail_campaign_1]
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )

    # This campaign will override the customer conversion goals.
    # Retrieve the current list of customer conversion goals.
    customer_conversion_goals: List[
        Dict[
            str,
            Union[
                ConversionActionCategoryEnum.ConversionActionCategory,
                ConversionOriginEnum.ConversionOrigin,
            ],
        ]
    ] = get_customer_conversion_goals(client, customer_id)

    # Performance Max campaigns require that repeated assets such as headlines
    # and descriptions be created before the campaign.
    # For the list of required assets for a Performance Max campaign, see
    # https://developers.google.com/google-ads/api/docs/performance-max/assets
    #
    # Create the headlines.
    headline_asset_resource_names: List[str] = create_multiple_text_assets(
        client,
        customer_id,
        [
            "Travel",
            "Travel Reviews",
            "Book travel",
        ],
    )
    # Create the descriptions.
    description_asset_resource_names: List[str] = create_multiple_text_assets(
        client,
        customer_id,
        [
            "Take to the air!",
            "Fly to the sky!",
        ],
    )

    # The below methods create and return MutateOperations that we later
    # provide to the GoogleAdsService.Mutate method in order to create the
    # entities in a single request. Since the entities for a Performance Max
    # campaign are closely tied to one-another, it's considered a best practice
    # to create them in a single Mutate request so they all complete
    # successfully or fail entirely, leaving no orphaned entities. See:
    # https://developers.google.com/google-ads/api/docs/mutating/overview
    campaign_budget_operation: MutateOperation = (
        create_campaign_budget_operation(
            client,
            customer_id,
        )
    )
    performance_max_campaign_operation: MutateOperation = (
        create_performance_max_campaign_operation(
            client,
            customer_id,
            merchant_center_account_id,
            brand_guidelines_enabled,
        )
    )
    campaign_criterion_operations: List[MutateOperation] = (
        create_campaign_criterion_operations(
            client,
            customer_id,
        )
    )
    asset_group_operation: MutateOperation = create_asset_group_operation(
        client, customer_id, final_url
    )
    listing_group_filter_operation: MutateOperation = (
        create_listing_group_filter_operation(client, customer_id)
    )
    asset_and_asset_group_asset_operations: List[MutateOperation] = (
        create_asset_and_asset_group_asset_operations(
            client,
            customer_id,
            headline_asset_resource_names,
            description_asset_resource_names,
            brand_guidelines_enabled,
        )
    )
    conversion_goal_operations: List[MutateOperation] = (
        create_conversion_goal_operations(
            client,
            customer_id,
            customer_conversion_goals,
        )
    )

    # Send the operations in a single Mutate request.
    response: MutateGoogleAdsResponse = googleads_service.mutate(
        customer_id=customer_id,
        mutate_operations=[
            # It's important to create these entities in this order because
            # they depend on each other.
            campaign_budget_operation,
            performance_max_campaign_operation,
            # Expand the list of multiple operations into the list of
            # other mutate operations.
            *campaign_criterion_operations,
            asset_group_operation,
            listing_group_filter_operation,
            *asset_and_asset_group_asset_operations,
            *conversion_goal_operations,
        ],
    )
    print_response_details(response)
    # [END add_performance_max_retail_campaign_1]


# [START add_performance_max_retail_campaign_2]
def create_campaign_budget_operation(
    client: GoogleAdsClient,
    customer_id: str,
) -> MutateOperation:
    """Creates a MutateOperation that creates a new CampaignBudget.

    A temporary ID will be assigned to this campaign budget so that it can be
    referenced by other objects being created in the same Mutate request.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        a MutateOperation that creates a CampaignBudget.
    """
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    campaign_budget_operation = mutate_operation.campaign_budget_operation
    campaign_budget: CampaignBudget = campaign_budget_operation.create
    campaign_budget.name = f"Performance Max retail campaign budget #{uuid4()}"
    # The budget period already defaults to DAILY.
    campaign_budget.amount_micros = 50000000
    campaign_budget.delivery_method = (
        client.enums.BudgetDeliveryMethodEnum.STANDARD
    )
    # A Performance Max campaign cannot use a shared campaign budget.
    campaign_budget.explicitly_shared = False

    # Set a temporary ID in the budget's resource name so it can be referenced
    # by the campaign in later steps.
    campaign_budget_service: CampaignBudgetServiceClient = client.get_service(
        "CampaignBudgetService"
    )
    campaign_budget.resource_name = (
        campaign_budget_service.campaign_budget_path(
            customer_id, _BUDGET_TEMPORARY_ID
        )
    )

    return mutate_operation
    # [END add_performance_max_retail_campaign_2]


# [START add_performance_max_retail_campaign_3]
def create_performance_max_campaign_operation(
    client: GoogleAdsClient,
    customer_id: str,
    merchant_center_account_id: int,
    brand_guidelines_enabled: bool,
) -> MutateOperation:
    """Creates a MutateOperation that creates a new Performance Max campaign.

    A temporary ID will be assigned to this campaign so that it can
    be referenced by other objects being created in the same Mutate request.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        merchant_center_account_id: The Merchant Center account ID.
        brand_guidelines_enabled: a boolean value indicating if the campaign is
          enabled for brand guidelines.

    Returns:
        a MutateOperation that creates a campaign.
    """
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    campaign: Campaign = mutate_operation.campaign_operation.create
    campaign.name = f"Performance Max retail campaign #{uuid4()}"
    # Set the campaign status as PAUSED. The campaign is the only entity in
    # the mutate request that should have its status set.
    campaign.status = client.enums.CampaignStatusEnum.PAUSED
    # All Performance Max campaigns have an advertising_channel_type of
    # PERFORMANCE_MAX. The advertising_channel_sub_type should not be set.
    campaign.advertising_channel_type = (
        client.enums.AdvertisingChannelTypeEnum.PERFORMANCE_MAX
    )
    # Bidding strategy must be set directly on the campaign.
    # Setting a portfolio bidding strategy by resource name is not supported.
    # Max Conversion and Max Conversion Value are the only strategies supported
    # for Performance Max campaigns.
    # An optional ROAS (Return on Advertising Spend) can be set for
    # maximize_conversion_value. The ROAS value must be specified as a ratio in
    # the API. It is calculated by dividing "total value" by "total spend".
    # For more information on Max Conversion Value, see the support article:
    # http://support.google.com/google-ads/answer/7684216.
    # A target_roas of 3.5 corresponds to a 350% return on ad spend.
    # campaign.maximize_conversion_value.target_roas = 3.5
    # For first time users, it's recommended not to set a target ROAS.
    # Although target ROAS is optional, you still need to define it
    # even if you do not want to use it.
    campaign.maximize_conversion_value.target_roas = None
    # Below is what you would use if you want to maximize conversions
    # campaign.maximize_conversions.target_cpa_micros = None
    # The target CPA is optional. This is the average amount that you would
    # like to spend per conversion action.

    # Set the shopping settings.
    campaign.shopping_setting.merchant_id = merchant_center_account_id

    # Optional: To use products only from a specific feed, set
    # shopping_setting.feed_label to the feed label used in Merchant Center.
    # See: https://support.google.com/merchants/answer/12453549.
    # Omitting the shopping_setting.feed_label field will use products from all
    # feeds.
    # campaign.shopping_setting.feed_label = "INSERT_FEED_LABEL_HERE"

    # Set if the campaign is enabled for brand guidelines. For more information
    # on brand guidelines, see https://support.google.com/google-ads/answer/14934472.
    campaign.brand_guidelines_enabled = brand_guidelines_enabled

    # Assign the resource name with a temporary ID.
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )
    campaign.resource_name = campaign_service.campaign_path(
        customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
    )
    # Set the budget using the given budget resource name.
    campaign_budget_service: CampaignBudgetServiceClient = client.get_service(
        "CampaignBudgetService"
    )
    campaign.campaign_budget = campaign_budget_service.campaign_budget_path(
        customer_id, _BUDGET_TEMPORARY_ID
    )

    # Declare whether or not this campaign serves political ads targeting the
    # EU. Valid values are:
    #   CONTAINS_EU_POLITICAL_ADVERTISING
    #   DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    campaign.contains_eu_political_advertising = (
        client.enums.EuPoliticalAdvertisingStatusEnum.DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    )

    # Optional fields
    campaign.start_date = (datetime.now() + timedelta(1)).strftime("%Y%m%d")
    campaign.end_date = (datetime.now() + timedelta(365)).strftime("%Y%m%d")

    # Configures the optional opt-in/out status for asset automation settings.
    for asset_automation_type_enum in [
        client.enums.AssetAutomationTypeEnum.GENERATE_IMAGE_EXTRACTION,
        client.enums.AssetAutomationTypeEnum.FINAL_URL_EXPANSION_TEXT_ASSET_AUTOMATION,
        client.enums.AssetAutomationTypeEnum.TEXT_ASSET_AUTOMATION,
        client.enums.AssetAutomationTypeEnum.GENERATE_ENHANCED_YOUTUBE_VIDEOS,
        client.enums.AssetAutomationTypeEnum.GENERATE_IMAGE_ENHANCEMENT
    ]:
        asset_automattion_setting: Campaign.AssetAutomationSetting = client.get_type("Campaign").AssetAutomationSetting()
        asset_automattion_setting.asset_automation_type = asset_automation_type_enum
        asset_automattion_setting.asset_automation_status = client.enums.AssetAutomationStatusEnum.OPTED_IN
        campaign.asset_automation_settings.append(asset_automattion_setting)

    return mutate_operation
    # [END add_performance_max_retail_campaign_3]


# [START add_performance_max_retail_campaign_4]
def create_campaign_criterion_operations(
    client: GoogleAdsClient,
    customer_id: str,
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that create new campaign criteria.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        a list of MutateOperations that create new campaign criteria.
    """
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )
    geo_target_constant_service: GeoTargetConstantServiceClient = (
        client.get_service("GeoTargetConstantService")
    )
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )

    operations: List[MutateOperation] = []
    # Set the LOCATION campaign criteria.
    # Target all of New York City except Brooklyn.
    # Location IDs are listed here:
    # https://developers.google.com/google-ads/api/reference/data/geotargets
    # and they can also be retrieved using the GeoTargetConstantService as shown
    # here: https://developers.google.com/google-ads/api/docs/targeting/location-targeting
    mutate_operation_nyc: MutateOperation = client.get_type("MutateOperation")
    campaign_criterion_nyc: CampaignCriterion = (
        mutate_operation_nyc.campaign_criterion_operation.create
    )
    campaign_criterion_nyc.campaign = campaign_service.campaign_path(
        customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
    )
    # Adds one positive location target for New York City (ID=1023191),
    # specifically adding the positive criteria before the negative one.
    campaign_criterion_nyc.location.geo_target_constant = (
        geo_target_constant_service.geo_target_constant_path("1023191")
    )
    campaign_criterion_nyc.negative = False
    operations.append(mutate_operation_nyc)

    # Next add the negative target for Brooklyn (ID=1022762).
    mutate_operation_brooklyn: MutateOperation = client.get_type(
        "MutateOperation"
    )
    campaign_criterion_brooklyn: CampaignCriterion = (
        mutate_operation_brooklyn.campaign_criterion_operation.create
    )
    campaign_criterion_brooklyn.campaign = campaign_service.campaign_path(
        customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
    )
    campaign_criterion_brooklyn.location.geo_target_constant = (
        geo_target_constant_service.geo_target_constant_path("1022762")
    )
    campaign_criterion_brooklyn.negative = True
    operations.append(mutate_operation_brooklyn)

    # Set the LANGUAGE campaign criterion.
    mutate_operation_lang: MutateOperation = client.get_type("MutateOperation")
    campaign_criterion_lang: CampaignCriterion = (
        mutate_operation_lang.campaign_criterion_operation.create
    )
    campaign_criterion_lang.campaign = campaign_service.campaign_path(
        customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
    )
    # Set the language.
    # For a list of all language codes, see:
    # https://developers.google.com/google-ads/api/reference/data/codes-formats#expandable-7
    campaign_criterion_lang.language.language_constant = (
        googleads_service.language_constant_path("1000")
    )  # English
    operations.append(mutate_operation_lang)

    return operations
    # [END add_performance_max_retail_campaign_4]


# [START add_performance_max_retail_campaign_5]
def create_multiple_text_assets(
    client: GoogleAdsClient, customer_id: str, texts: List[str]
) -> List[str]:
    """Creates multiple text assets and returns the list of resource names.

    These repeated assets must be created in a separate request prior to
    creating the campaign.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        texts: a list of strings, each of which will be used to create a text
          asset.

    Returns:
        asset_resource_names: a list of asset resource names.
    """
    # Here again we use the GoogleAdService to create multiple text
    # assets in a single request.
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )

    operations: List[MutateOperation] = []
    for text_content in texts:
        mutate_operation: MutateOperation = client.get_type("MutateOperation")
        asset: Asset = mutate_operation.asset_operation.create
        asset.text_asset.text = text_content
        operations.append(mutate_operation)

    # Send the operations in a single Mutate request.
    response: MutateGoogleAdsResponse = googleads_service.mutate(
        customer_id=customer_id,
        mutate_operations=operations,
    )
    asset_resource_names: List[str] = []
    for result in response.mutate_operation_responses:
        if result._pb.HasField("asset_result"):
            asset_resource_names.append(result.asset_result.resource_name)
    print_response_details(response)
    return asset_resource_names
    # [END add_performance_max_retail_campaign_5]


# [START add_performance_max_retail_campaign_10]
def create_asset_group_operation(
    client: GoogleAdsClient, customer_id: str, final_url: str
) -> MutateOperation:
    """Creates a MutateOperation that creates a new asset group.

    A temporary ID will be assigned to this asset group so that it can
    be referenced by other objects being created in the same Mutate request.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        final_url: the final URL.

    Returns:
        a MutateOperation that creates a new asset group.
    """
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )
    asset_group_service: AssetGroupServiceClient = client.get_service(
        "AssetGroupService"
    )
    # Create the AssetGroup.
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    asset_group: AssetGroup = mutate_operation.asset_group_operation.create
    asset_group.name = f"Performance Max retail asset group #{uuid4()}"
    asset_group.campaign = campaign_service.campaign_path(
        customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
    )
    asset_group.final_urls.append(final_url)
    asset_group.final_mobile_urls.append(final_url)
    asset_group.status = client.enums.AssetGroupStatusEnum.PAUSED
    asset_group.resource_name = asset_group_service.asset_group_path(
        customer_id,
        _ASSET_GROUP_TEMPORARY_ID,
    )

    return mutate_operation
    # [END add_performance_max_retail_campaign_10]


# [START add_performance_max_retail_campaign_11]
def create_listing_group_filter_operation(
    client: GoogleAdsClient, customer_id: str
) -> MutateOperation:
    """Creates a MutateOperation that creates a new listing group filter.

    A temporary ID will be assigned to this listing group filter so that it
    can be referenced by other objects being created in the same Mutate request.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        a MutateOperation that creates a new listing group filter.
    """
    asset_group_service: AssetGroupServiceClient = client.get_service(
        "AssetGroupService"
    )
    # Creates a new ad group criterion containing the "default" listing
    # group (All products).
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    asset_group_listing_group: AssetGroupListingGroupFilter = (
        mutate_operation.asset_group_listing_group_filter_operation.create
    )
    asset_group_listing_group.asset_group = (
        asset_group_service.asset_group_path(
            customer_id,
            _ASSET_GROUP_TEMPORARY_ID,
        )
    )
    asset_group_listing_group.type_ = (
        client.enums.ListingGroupFilterTypeEnum.UNIT_INCLUDED
    )
    # Because this is a Performance Max campaign for retail, we need to specify
    # that this is in the shopping listing source.
    asset_group_listing_group.listing_source = (
        client.enums.ListingGroupFilterListingSourceEnum.SHOPPING
    )

    return mutate_operation
    # [END add_performance_max_retail_campaign_11]


# [START add_performance_max_retail_campaign_6]
def create_asset_and_asset_group_asset_operations(
    client: GoogleAdsClient,
    customer_id: str,
    headline_asset_resource_names: List[str],
    description_asset_resource_names: List[str],
    brand_guidelines_enabled: bool,
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that create a new asset_group.

    A temporary ID will be assigned to this asset group so that it can
    be referenced by other objects being created in the same Mutate request.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        headline_asset_resource_names: a list of headline resource names.
        description_asset_resource_names: a list of description resource names.
        brand_guidelines_enabled: a boolean value indicating if the campaign is
          enabled for brand guidelines.

    Returns:
        MutateOperations that create a new asset group and related assets.
    """
    asset_group_service: AssetGroupServiceClient = client.get_service(
        "AssetGroupService"
    )

    operations: List[MutateOperation] = []

    # For the list of required assets for a Performance Max campaign, see
    # https://developers.google.com/google-ads/api/docs/performance-max/assets

    # An AssetGroup is linked to an Asset by creating a new AssetGroupAsset
    # and providing:
    #   the resource name of the AssetGroup
    #   the resource name of the Asset
    #   the field_type of the Asset in this AssetGroup.
    #
    # To learn more about AssetGroups, see
    # https://developers.google.com/google-ads/api/docs/performance-max/asset-groups

    # Link the previously created multiple text assets.

    # Link the headline assets.
    for resource_name in headline_asset_resource_names:
        mutate_operation: MutateOperation = client.get_type("MutateOperation")
        asset_group_asset: AssetGroupAsset = (
            mutate_operation.asset_group_asset_operation.create
        )
        asset_group_asset.field_type = client.enums.AssetFieldTypeEnum.HEADLINE
        asset_group_asset.asset_group = asset_group_service.asset_group_path(
            customer_id,
            _ASSET_GROUP_TEMPORARY_ID,
        )
        asset_group_asset.asset = resource_name
        operations.append(mutate_operation)

    #  Link the description assets.
    for resource_name in description_asset_resource_names:
        mutate_operation: MutateOperation = client.get_type("MutateOperation")
        asset_group_asset: AssetGroupAsset = (
            mutate_operation.asset_group_asset_operation.create
        )
        asset_group_asset.field_type = (
            client.enums.AssetFieldTypeEnum.DESCRIPTION
        )
        asset_group_asset.asset_group = asset_group_service.asset_group_path(
            customer_id,
            _ASSET_GROUP_TEMPORARY_ID,
        )
        asset_group_asset.asset = resource_name
        operations.append(mutate_operation)

    # Create and link the long headline text asset.
    mutate_operations_long_headline: List[MutateOperation] = (
        create_and_link_text_asset(
            client,
            customer_id,
            "Travel the World",
            client.enums.AssetFieldTypeEnum.LONG_HEADLINE,
        )
    )
    operations.extend(mutate_operations_long_headline)

    # Create and link the business name and logo asset.
    mutate_operations_brand: List[MutateOperation] = (
        create_and_link_brand_assets(
            client,
            customer_id,
            brand_guidelines_enabled,
            "Interplanetary Cruises",
            "https://gaagl.page.link/1Crm",
            "Logo Image",
        )
    )
    operations.extend(mutate_operations_brand)

    # Create and link the image assets.

    # Create and link the Marketing Image Asset.
    mutate_operations_marketing_image: List[MutateOperation] = (
        create_and_link_image_asset(
            client,
            customer_id,
            "https://gaagl.page.link/Eit5",
            client.enums.AssetFieldTypeEnum.MARKETING_IMAGE,
            "Marketing Image",
        )
    )
    operations.extend(mutate_operations_marketing_image)

    # Create and link the Square Marketing Image Asset.
    mutate_operations_square_image: List[MutateOperation] = (
        create_and_link_image_asset(
            client,
            customer_id,
            "https://gaagl.page.link/bjYi",
            client.enums.AssetFieldTypeEnum.SQUARE_MARKETING_IMAGE,
            "Square Marketing Image",
        )
    )
    operations.extend(mutate_operations_square_image)

    # After being created the list must be sorted so that all asset
    # operations come before all the asset group asset operations,
    # otherwise the API will reject the request.
    return sort_asset_and_asset_group_asset_operations(operations)
    # [END add_performance_max_retail_campaign_6]


# [START add_performance_max_retail_campaign_7]
def create_and_link_text_asset(
    client: GoogleAdsClient,
    customer_id: str,
    text: str,
    field_type: AssetFieldTypeEnum.AssetFieldType,
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that create a new linked text asset.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        text: the text of the asset to be created.
        field_type: the field_type of the new asset in the AssetGroupAsset.

    Returns:
        MutateOperations that create a new linked text asset.
    """
    global _next_temp_id
    operations: List[MutateOperation] = []
    asset_service: AssetServiceClient = client.get_service("AssetService")
    asset_group_service: AssetGroupServiceClient = client.get_service(
        "AssetGroupService"
    )

    # Create the Text Asset.
    asset_temp_resource_name = asset_service.asset_path(
        customer_id, str(_next_temp_id)
    )
    mutate_operation_asset: MutateOperation = client.get_type("MutateOperation")
    asset: Asset = mutate_operation_asset.asset_operation.create
    asset.resource_name = asset_temp_resource_name
    asset.text_asset.text = text
    operations.append(mutate_operation_asset)

    # Create an AssetGroupAsset to link the Asset to the AssetGroup.
    mutate_operation_group_asset: MutateOperation = client.get_type(
        "MutateOperation"
    )
    asset_group_asset: AssetGroupAsset = (
        mutate_operation_group_asset.asset_group_asset_operation.create
    )
    asset_group_asset.field_type = field_type
    asset_group_asset.asset_group = asset_group_service.asset_group_path(
        customer_id,
        _ASSET_GROUP_TEMPORARY_ID,
    )
    asset_group_asset.asset = asset_temp_resource_name
    operations.append(mutate_operation_group_asset)

    _next_temp_id -= 1
    return operations
    # [END add_performance_max_retail_campaign_7]


# [START add_performance_max_retail_campaign_8]
def create_and_link_image_asset(
    client: GoogleAdsClient,
    customer_id: str,
    url: str,
    field_type: AssetFieldTypeEnum.AssetFieldType,
    asset_name: str,
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that create a new linked image asset.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        url: the url of the image to be retrieved and put into an asset.
        field_type: the field_type of the new asset in the AssetGroupAsset.
        asset_name: the asset name.

    Returns:
        MutateOperations that create a new linked image asset.
    """
    global _next_temp_id
    operations: List[MutateOperation] = []
    asset_service: AssetServiceClient = client.get_service("AssetService")
    asset_group_service: AssetGroupServiceClient = client.get_service(
        "AssetGroupService"
    )

    # Create the Image Asset.
    asset_temp_resource_name = asset_service.asset_path(
        customer_id, str(_next_temp_id)
    )
    mutate_operation_asset: MutateOperation = client.get_type("MutateOperation")
    asset: Asset = mutate_operation_asset.asset_operation.create
    asset.resource_name = asset_temp_resource_name
    asset.type_ = client.enums.AssetTypeEnum.IMAGE
    # Provide a unique friendly name to identify your asset.
    # When there is an existing image asset with the same content but a different
    # name, the new name will be dropped silently.
    asset.name = asset_name
    asset.image_asset.data = get_image_bytes_from_url(url)
    operations.append(mutate_operation_asset)

    # Create an AssetGroupAsset to link the Asset to the AssetGroup.
    mutate_operation_group_asset: MutateOperation = client.get_type(
        "MutateOperation"
    )
    asset_group_asset: AssetGroupAsset = (
        mutate_operation_group_asset.asset_group_asset_operation.create
    )
    asset_group_asset.field_type = field_type
    asset_group_asset.asset_group = asset_group_service.asset_group_path(
        customer_id,
        _ASSET_GROUP_TEMPORARY_ID,
    )
    asset_group_asset.asset = asset_temp_resource_name
    operations.append(mutate_operation_group_asset)

    _next_temp_id -= 1
    return operations
    # [END add_performance_max_retail_campaign_8]


# [START add_performance_max_retail_campaign_12]
def sort_asset_and_asset_group_asset_operations(
    operations: List[MutateOperation],
) -> List[MutateOperation]:
    """Sorts a list of asset and asset group asset operations.

    This sorts the list such that all asset operations precede
    all asset group asset operations. If asset group assets are
    created before assets then an error will be returned by
    the API.

    Args:
        operations: a list of asset and asset group asset operations.

    Returns:
        a sorted list of asset and asset group asset operations.
    """

    def sorter(operation: MutateOperation) -> bool:
        """Determines whether the operation creates an asset group asset.

        Args:
            operation: a MutateOperation instance.

        Returns:
            True if the MutateOperation creates an asset group asset.
        """
        # Check if the oneof field 'asset_group_asset_operation' is set.
        return (
            operation.asset_group_asset_operation
            != type(operation.asset_group_asset_operation)()
        )

    return sorted(operations, key=sorter)
    # [END add_performance_max_retail_campaign_12]


# [START add_performance_max_retail_campaign_9]
def get_customer_conversion_goals(
    client: GoogleAdsClient, customer_id: str
) -> List[
    Dict[
        str,
        Union[
            ConversionActionCategoryEnum.ConversionActionCategory,
            ConversionOriginEnum.ConversionOrigin,
        ],
    ]
]:
    """Retrieves the list of customer conversion goals.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        a list of dicts containing the category and origin of customer
        conversion goals.
    """
    ga_service: GoogleAdsServiceClient = client.get_service("GoogleAdsService")
    customer_conversion_goals: List[
        Dict[
            str,
            Union[
                ConversionActionCategoryEnum.ConversionActionCategory,
                ConversionOriginEnum.ConversionOrigin,
            ],
        ]
    ] = []
    query: str = """
            SELECT
              customer_conversion_goal.category,
              customer_conversion_goal.origin
            FROM customer_conversion_goal
            """
    # The number of conversion goals is typically less than 50 so we use
    # GoogleAdsService.search instead of search_stream.
    search_request: SearchGoogleAdsRequest = client.get_type(
        "SearchGoogleAdsRequest"
    )
    search_request.customer_id = customer_id
    search_request.query = query
    results: SearchGoogleAdsResponse = ga_service.search(request=search_request)

    # Iterate over the results and build the list of conversion goals.
    for row in results:
        customer_conversion_goals.append(
            {
                "category": row.customer_conversion_goal.category,
                "origin": row.customer_conversion_goal.origin,
            }
        )
    return customer_conversion_goals


def create_conversion_goal_operations(
    client: GoogleAdsClient,
    customer_id: str,
    customer_conversion_goals: List[
        Dict[
            str,
            Union[
                ConversionActionCategoryEnum.ConversionActionCategory,
                ConversionOriginEnum.ConversionOrigin,
            ],
        ]
    ],
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that override customer conversion goals.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        customer_conversion_goals: the list of customer conversion goals that
          will be overridden.

    Returns:
        MutateOperations that update campaign conversion goals.
    """
    campaign_conversion_goal_service: CampaignConversionGoalServiceClient = (
        client.get_service("CampaignConversionGoalService")
    )
    operations: List[MutateOperation] = []

    # To override the customer conversion goals, we will change the
    # biddability of each of the customer conversion goals so that only
    # the desired conversion goal is biddable in this campaign.
    for customer_goal_dict in customer_conversion_goals:
        mutate_operation: MutateOperation = client.get_type("MutateOperation")
        campaign_conversion_goal: CampaignConversionGoal = (
            mutate_operation.campaign_conversion_goal_operation.update
        )

        category_enum_value: (
            ConversionActionCategoryEnum.ConversionActionCategory
        ) = customer_goal_dict["category"]
        origin_enum_value: ConversionOriginEnum.ConversionOrigin = (
            customer_goal_dict["origin"]
        )

        campaign_conversion_goal.resource_name = (
            campaign_conversion_goal_service.campaign_conversion_goal_path(
                customer_id,
                _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID,
                category_enum_value.name,
                origin_enum_value.name,
            )
        )
        # Change the biddability for the campaign conversion goal.
        # Set biddability to True for the desired (category, origin).
        # Set biddability to False for all other conversion goals.
        # Note:
        #  1- It is assumed that this Conversion Action
        #     (category=PURCHASE, origin=WEBSITE) exists in this account.
        #  2- More than one goal can be biddable if desired. This example
        #     shows only one.
        if (
            category_enum_value
            == client.enums.ConversionActionCategoryEnum.PURCHASE
            and origin_enum_value == client.enums.ConversionOriginEnum.WEBSITE
        ):
            biddable = True
        else:
            biddable = False
        campaign_conversion_goal.biddable = biddable
        field_mask = protobuf_helpers.field_mask(
            None, campaign_conversion_goal._pb
        )
        client.copy_from(
            mutate_operation.campaign_conversion_goal_operation.update_mask,
            field_mask,
        )
        operations.append(mutate_operation)

    return operations
    # [END add_performance_max_retail_campaign_9]


def create_and_link_brand_assets(
    client: GoogleAdsClient,
    customer_id: str,
    brand_guidelines_enabled: bool,
    business_name: str,
    logo_url: str,
    logo_name: str,
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that create linked brand assets.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        brand_guidelines_enabled: a boolean value indicating if the campaign is
          enabled for brand guidelines.
        business_name: the business name text to be put into an asset.
        logo_url: the url of the logo to be retrieved and put into an asset.
        logo_name: the asset name of the logo.

    Returns:
        MutateOperations that create linked brand assets.
    """
    global _next_temp_id
    operations: List[MutateOperation] = []
    asset_service: AssetServiceClient = client.get_service("AssetService")

    # Create the Text Asset.
    text_asset_temp_id: int = _next_temp_id
    _next_temp_id -= 1
    text_asset_resource_name = asset_service.asset_path(
        customer_id, str(text_asset_temp_id)
    )

    text_mutate_operation: MutateOperation = client.get_type("MutateOperation")
    text_asset_obj: Asset = text_mutate_operation.asset_operation.create
    text_asset_obj.resource_name = text_asset_resource_name
    text_asset_obj.text_asset.text = business_name
    operations.append(text_mutate_operation)

    # Create the Image Asset.
    image_asset_temp_id: int = _next_temp_id
    _next_temp_id -= 1
    image_asset_resource_name = asset_service.asset_path(
        customer_id, str(image_asset_temp_id)
    )

    image_mutate_operation: MutateOperation = client.get_type("MutateOperation")
    image_asset_obj: Asset = image_mutate_operation.asset_operation.create
    image_asset_obj.resource_name = image_asset_resource_name
    # Provide a unique friendly name to identify your asset.
    # When there is an existing image asset with the same content but a different
    # name, the new name will be dropped silently.
    image_asset_obj.name = logo_name
    image_asset_obj.type_ = client.enums.AssetTypeEnum.IMAGE
    image_asset_obj.image_asset.data = get_image_bytes_from_url(logo_url)
    operations.append(image_mutate_operation)

    if brand_guidelines_enabled:
        # Create CampaignAsset resources to link the Asset resources to the Campaign.
        campaign_service: CampaignServiceClient = client.get_service(
            "CampaignService"
        )

        business_name_ca_mutate_op: MutateOperation = client.get_type(
            "MutateOperation"
        )
        business_name_campaign_asset: CampaignAsset = (
            business_name_ca_mutate_op.campaign_asset_operation.create
        )
        business_name_campaign_asset.field_type = (
            client.enums.AssetFieldTypeEnum.BUSINESS_NAME
        )
        business_name_campaign_asset.campaign = campaign_service.campaign_path(
            customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
        )
        business_name_campaign_asset.asset = text_asset_resource_name
        operations.append(business_name_ca_mutate_op)

        logo_ca_mutate_op: MutateOperation = client.get_type("MutateOperation")
        logo_campaign_asset: CampaignAsset = (
            logo_ca_mutate_op.campaign_asset_operation.create
        )
        logo_campaign_asset.field_type = client.enums.AssetFieldTypeEnum.LOGO
        logo_campaign_asset.campaign = campaign_service.campaign_path(
            customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
        )
        logo_campaign_asset.asset = image_asset_resource_name
        operations.append(logo_ca_mutate_op)

    else:
        # Create AssetGroupAsset resources to link the Asset resources to the AssetGroup.
        asset_group_service: AssetGroupServiceClient = client.get_service(
            "AssetGroupService"
        )

        business_name_aga_mutate_op: MutateOperation = client.get_type(
            "MutateOperation"
        )
        business_name_asset_group_asset: AssetGroupAsset = (
            business_name_aga_mutate_op.asset_group_asset_operation.create
        )
        business_name_asset_group_asset.field_type = (
            client.enums.AssetFieldTypeEnum.BUSINESS_NAME
        )
        business_name_asset_group_asset.asset_group = (
            asset_group_service.asset_group_path(
                customer_id,
                _ASSET_GROUP_TEMPORARY_ID,
            )
        )
        business_name_asset_group_asset.asset = text_asset_resource_name
        operations.append(business_name_aga_mutate_op)

        logo_aga_mutate_op: MutateOperation = client.get_type("MutateOperation")
        logo_asset_group_asset: AssetGroupAsset = (
            logo_aga_mutate_op.asset_group_asset_operation.create
        )
        logo_asset_group_asset.field_type = client.enums.AssetFieldTypeEnum.LOGO
        logo_asset_group_asset.asset_group = (
            asset_group_service.asset_group_path(
                customer_id,
                _ASSET_GROUP_TEMPORARY_ID,
            )
        )
        logo_asset_group_asset.asset = image_asset_resource_name
        operations.append(logo_aga_mutate_op)

    return operations


def print_response_details(response: MutateGoogleAdsResponse) -> None:
    """Prints the details of a MutateGoogleAdsResponse.

    Parses the "response" oneof field name and uses it to extract the new
    entity's name and resource name.

    Args:
        response: a MutateGoogleAdsResponse object.
    """
    # Parse the Mutate response to print details about the entities that
    # were created by the request.
    suffix: str = "_result"
    for result_item in response.mutate_operation_responses:
        # Ensure result_item is MutateOperationResponse, not just Any
        result_pb: MutateOperationResponse = result_item
        for field_descriptor, value in result_pb._pb.ListFields():
            field_name_str: str = field_descriptor.name
            if field_name_str.endswith(suffix):
                name = field_name_str[: -len(suffix)]
            else:
                name = field_name_str
            print(
                f"Created a(n) {convert_snake_case_to_upper_case(name)} with "
                f"{str(value).strip()}."
            )


# [END add_performance_max_retail_campaign]

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=("Creates a Performance Max retail campaign.")
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-m",
        "--merchant_center_account_id",
        type=int,
        required=True,
        help="The Merchant Center account ID.",
    )
    parser.add_argument(
        "-u",
        "--final_url",
        type=str,
        required=False,
        default="http://www.example.com",
        help="The final URL for the asset group of the campaign.",
    )
    parser.add_argument(
        "-b",
        "--brand_guidelines_enabled",
        type=bool,
        default=False,
        help=(
            "A boolean value indicating if the created campaign is enabled "
            "for brand guidelines."
        ),
    )

    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.merchant_center_account_id,
            args.final_url,
            args.brand_guidelines_enabled,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "VideoAdSequenceInteractionTypeEnum",
    },
)


class VideoAdSequenceInteractionTypeEnum(proto.Message):
    r"""Container for enum describing an interaction between a viewer
    and a video in a video ad sequence.

    """

    class VideoAdSequenceInteractionType(proto.Enum):
        r"""Enum describing an interaction between a viewer and a video
        in a video ad sequence.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            PAID_VIEW (2):
                The viewer engaged with or watched at least
                30 seconds of the ad (or the entire ad, if it
                was less than 30 seconds). Only available for
                campaigns using Target CPM as the bidding
                strategy and skippable in-stream ads as the ad
                format.
            SKIP (3):
                The viewer skipped the ad. Only available for
                campaigns using Target CPM as the bidding
                strategy and skippable in-stream ads as the ad
                format.
            IMPRESSION (4):
                The ad was shown to the viewer.
            ENGAGED_IMPRESSION (5):
                An ad impression that was not immediately
                skipped, but didn't reach the billable event
                either.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        PAID_VIEW = 2
        SKIP = 3
        IMPRESSION = 4
        ENGAGED_IMPRESSION = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import hotel_reconciliation_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "HotelReconciliation",
    },
)


class HotelReconciliation(proto.Message):
    r"""A hotel reconciliation. It contains conversion information
    from Hotel bookings to reconcile with advertiser records. These
    rows may be updated or canceled before billing through Bulk
    Uploads.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the hotel reconciliation.
            Hotel reconciliation resource names have the form:

            ``customers/{customer_id}/hotelReconciliations/{commission_id}``
        commission_id (str):
            Required. Output only. The commission ID is
            Google's ID for this booking. Every booking
            event is assigned a Commission ID to help you
            match it to a guest stay.
        order_id (str):
            Output only. The order ID is the identifier for this booking
            as provided in the 'transaction_id' parameter of the
            conversion tracking tag.
        campaign (str):
            Output only. The resource name for the
            Campaign associated with the conversion.
        hotel_center_id (int):
            Output only. Identifier for the Hotel Center
            account which provides the rates for the Hotel
            campaign.
        hotel_id (str):
            Output only. Unique identifier for the booked
            property, as provided in the Hotel Center feed.
            The hotel ID comes from the 'ID' parameter of
            the conversion tracking tag.
        check_in_date (str):
            Output only. Check-in date recorded when the
            booking is made. If the check-in date is
            modified at reconciliation, the revised date
            will then take the place of the original date in
            this column. Format is YYYY-MM-DD.
        check_out_date (str):
            Output only. Check-out date recorded when the
            booking is made. If the check-in date is
            modified at reconciliation, the revised date
            will then take the place of the original date in
            this column. Format is YYYY-MM-DD.
        reconciled_value_micros (int):
            Required. Output only. Reconciled value is
            the final value of a booking as paid by the
            guest. If original booking value changes for any
            reason, such as itinerary changes or room
            upsells, the reconciled value should be the full
            final amount collected. If a booking is
            canceled, the reconciled value should include
            the value of any cancellation fees or
            non-refundable nights charged. Value is in
            millionths of the base unit currency. For
            example, $12.35 would be represented as
            12350000. Currency unit is in the default
            customer currency.
        billed (bool):
            Output only. Whether a given booking has been
            billed. Once billed, a booking can't be
            modified.
        status (google.ads.googleads.v20.enums.types.HotelReconciliationStatusEnum.HotelReconciliationStatus):
            Required. Output only. Current status of a
            booking with regards to reconciliation and
            billing. Bookings should be reconciled within 45
            days after the check-out date. Any booking not
            reconciled within 45 days will be billed at its
            original value.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    commission_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    order_id: str = proto.Field(
        proto.STRING,
        number=3,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=11,
    )
    hotel_center_id: int = proto.Field(
        proto.INT64,
        number=4,
    )
    hotel_id: str = proto.Field(
        proto.STRING,
        number=5,
    )
    check_in_date: str = proto.Field(
        proto.STRING,
        number=6,
    )
    check_out_date: str = proto.Field(
        proto.STRING,
        number=7,
    )
    reconciled_value_micros: int = proto.Field(
        proto.INT64,
        number=8,
    )
    billed: bool = proto.Field(
        proto.BOOL,
        number=9,
    )
    status: (
        hotel_reconciliation_status.HotelReconciliationStatusEnum.HotelReconciliationStatus
    ) = proto.Field(
        proto.ENUM,
        number=10,
        enum=hotel_reconciliation_status.HotelReconciliationStatusEnum.HotelReconciliationStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "PlacementTypeEnum",
    },
)


class PlacementTypeEnum(proto.Message):
    r"""Container for enum describing possible placement types."""

    class PlacementType(proto.Enum):
        r"""Possible placement types for a feed mapping.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            WEBSITE (2):
                Websites(for example,
                'www.flowers4sale.com').
            MOBILE_APP_CATEGORY (3):
                Mobile application categories(for example,
                'Games').
            MOBILE_APPLICATION (4):
                mobile applications(for example,
                'mobileapp::2-com.whatsthewordanswers').
            YOUTUBE_VIDEO (5):
                YouTube videos(for example,
                'youtube.com/video/wtLJPvx7-ys').
            YOUTUBE_CHANNEL (6):
                YouTube channels(for example,
                'youtube.com::L8ZULXASCc1I_oaOT0NaOQ').
            GOOGLE_PRODUCTS (7):
                Surfaces owned and operated by Google(for
                example, 'tv.google.com').
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        WEBSITE = 2
        MOBILE_APP_CATEGORY = 3
        MOBILE_APPLICATION = 4
        YOUTUBE_VIDEO = 5
        YOUTUBE_CHANNEL = 6
        GOOGLE_PRODUCTS = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "RecommendationTypeEnum",
    },
)


class RecommendationTypeEnum(proto.Message):
    r"""Container for enum describing types of recommendations."""

    class RecommendationType(proto.Enum):
        r"""Types of recommendations.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            CAMPAIGN_BUDGET (2):
                Provides optimized budget recommendations for
                campaigns.
            KEYWORD (3):
                Keyword recommendation.
            TEXT_AD (4):
                Recommendation to add a new text ad.
            TARGET_CPA_OPT_IN (5):
                Recommendation to update a campaign to use a
                Target CPA bidding strategy.
            MAXIMIZE_CONVERSIONS_OPT_IN (6):
                Recommendation to update a campaign to use
                the Maximize Conversions bidding strategy.
            ENHANCED_CPC_OPT_IN (7):
                Recommendation to enable Enhanced Cost Per
                Click for a campaign.
            SEARCH_PARTNERS_OPT_IN (8):
                Recommendation to start showing your
                campaign's ads on Google Search Partners
                Websites.
            MAXIMIZE_CLICKS_OPT_IN (9):
                Recommendation to update a campaign to use a
                Maximize Clicks bidding strategy.
            OPTIMIZE_AD_ROTATION (10):
                Recommendation to start using the "Optimize"
                ad rotation setting for the given ad group.
            KEYWORD_MATCH_TYPE (14):
                Recommendation to change an existing keyword
                from one match type to a broader match type.
            MOVE_UNUSED_BUDGET (15):
                Recommendation to move unused budget from one
                budget to a constrained budget.
            FORECASTING_CAMPAIGN_BUDGET (16):
                Budget recommendation for campaigns that are expected to
                become budget-constrained in the future (as opposed to the
                CAMPAIGN_BUDGET recommendation, which applies to campaigns
                that are currently budget-constrained).
            TARGET_ROAS_OPT_IN (17):
                Recommendation to update a campaign to use a
                Target ROAS bidding strategy.
            RESPONSIVE_SEARCH_AD (18):
                Recommendation to add a new responsive search
                ad.
            MARGINAL_ROI_CAMPAIGN_BUDGET (19):
                Budget recommendation for campaigns whose ROI
                is predicted to increase with a budget
                adjustment.
            USE_BROAD_MATCH_KEYWORD (20):
                Recommendation to add broad match versions of
                keywords for fully automated conversion-based
                bidding campaigns.
            RESPONSIVE_SEARCH_AD_ASSET (21):
                Recommendation to add new responsive search
                ad assets.
            UPGRADE_SMART_SHOPPING_CAMPAIGN_TO_PERFORMANCE_MAX (22):
                Recommendation to upgrade a Smart Shopping
                campaign to a Performance Max campaign.
            RESPONSIVE_SEARCH_AD_IMPROVE_AD_STRENGTH (23):
                Recommendation to improve strength of
                responsive search ad.
            DISPLAY_EXPANSION_OPT_IN (24):
                Recommendation to update a campaign to use
                Display Expansion.
            UPGRADE_LOCAL_CAMPAIGN_TO_PERFORMANCE_MAX (25):
                Recommendation to upgrade a Local campaign to
                a Performance Max campaign.
            RAISE_TARGET_CPA_BID_TOO_LOW (26):
                Recommendation to raise target CPA when it is
                too low and there are very few or no
                conversions. It is applied asynchronously and
                can take minutes depending on the number of ad
                groups there are in the related campaign.
            FORECASTING_SET_TARGET_ROAS (27):
                Recommendation to raise the budget in advance
                of a seasonal event that is forecasted to
                increase traffic, and change bidding strategy
                from maximize conversion value to target ROAS.
            CALLOUT_ASSET (28):
                Recommendation to add callout assets to
                campaign or customer level.
            SITELINK_ASSET (29):
                Recommendation to add sitelink assets to
                campaign or customer level.
            CALL_ASSET (30):
                Recommendation to add call assets to campaign
                or customer level.
            SHOPPING_ADD_AGE_GROUP (31):
                Recommendation to add the age group attribute
                to offers that are demoted because of a missing
                age group.
            SHOPPING_ADD_COLOR (32):
                Recommendation to add a color to offers that
                are demoted because of a missing color.
            SHOPPING_ADD_GENDER (33):
                Recommendation to add a gender to offers that
                are demoted because of a missing gender.
            SHOPPING_ADD_GTIN (34):
                Recommendation to add a GTIN (Global Trade
                Item Number) to offers that are demoted because
                of a missing GTIN.
            SHOPPING_ADD_MORE_IDENTIFIERS (35):
                Recommendation to add more identifiers to
                offers that are demoted because of missing
                identifiers.
            SHOPPING_ADD_SIZE (36):
                Recommendation to add the size to offers that
                are demoted because of a missing size.
            SHOPPING_ADD_PRODUCTS_TO_CAMPAIGN (37):
                Recommendation informing a customer about a
                campaign that cannot serve because no products
                are being targeted.
            SHOPPING_FIX_DISAPPROVED_PRODUCTS (38):
                The shopping recommendation informing a
                customer about campaign with a high percentage
                of disapproved products.
            SHOPPING_TARGET_ALL_OFFERS (39):
                Recommendation to create a catch-all campaign
                that targets all offers.
            SHOPPING_FIX_SUSPENDED_MERCHANT_CENTER_ACCOUNT (40):
                Recommendation to fix Merchant Center account
                suspension issues.
            SHOPPING_FIX_MERCHANT_CENTER_ACCOUNT_SUSPENSION_WARNING (41):
                Recommendation to fix Merchant Center account
                suspension warning issues.
            SHOPPING_MIGRATE_REGULAR_SHOPPING_CAMPAIGN_OFFERS_TO_PERFORMANCE_MAX (42):
                Recommendation to migrate offers targeted by
                Regular Shopping Campaigns to existing
                Performance Max campaigns.
            DYNAMIC_IMAGE_EXTENSION_OPT_IN (43):
                Recommendation to enable dynamic image
                extensions on the account, allowing Google to
                find the best images from ad landing pages and
                complement text ads.
            RAISE_TARGET_CPA (44):
                Recommendation to raise Target CPA based on
                Google predictions modeled from past
                conversions. It is applied asynchronously and
                can take minutes depending on the number of ad
                groups there are in the related campaign.
            LOWER_TARGET_ROAS (45):
                Recommendation to lower Target ROAS.
            PERFORMANCE_MAX_OPT_IN (46):
                Recommendation to opt into Performance Max
                campaigns.
            IMPROVE_PERFORMANCE_MAX_AD_STRENGTH (47):
                Recommendation to improve the asset group
                strength of a Performance Max campaign to an
                "Excellent" rating.
            MIGRATE_DYNAMIC_SEARCH_ADS_CAMPAIGN_TO_PERFORMANCE_MAX (48):
                Recommendation to migrate Dynamic Search Ads
                to Performance Max campaigns.
            FORECASTING_SET_TARGET_CPA (49):
                Recommendation to set a target CPA for
                campaigns that do not have one specified, in
                advance of a seasonal event that is forecasted
                to increase traffic.
            SET_TARGET_CPA (50):
                Recommendation to set a target CPA for
                campaigns that do not have one specified.
            SET_TARGET_ROAS (51):
                Recommendation to set a target ROAS for
                campaigns that do not have one specified.
            MAXIMIZE_CONVERSION_VALUE_OPT_IN (52):
                Recommendation to update a campaign to use
                the Maximize Conversion Value bidding strategy.
            IMPROVE_GOOGLE_TAG_COVERAGE (53):
                Recommendation to deploy Google Tag on more
                pages.
            PERFORMANCE_MAX_FINAL_URL_OPT_IN (54):
                Recommendation to turn on Final URL expansion
                for your Performance Max campaigns.
            REFRESH_CUSTOMER_MATCH_LIST (55):
                Recommendation to update a customer list that
                hasn't been updated in the last 90 days.
            CUSTOM_AUDIENCE_OPT_IN (56):
                Recommendation to create a custom audience.
            LEAD_FORM_ASSET (57):
                Recommendation to add lead form assets to
                campaign or customer level.
            IMPROVE_DEMAND_GEN_AD_STRENGTH (58):
                Recommendation to improve the strength of ads
                in Demand Gen campaigns.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CAMPAIGN_BUDGET = 2
        KEYWORD = 3
        TEXT_AD = 4
        TARGET_CPA_OPT_IN = 5
        MAXIMIZE_CONVERSIONS_OPT_IN = 6
        ENHANCED_CPC_OPT_IN = 7
        SEARCH_PARTNERS_OPT_IN = 8
        MAXIMIZE_CLICKS_OPT_IN = 9
        OPTIMIZE_AD_ROTATION = 10
        KEYWORD_MATCH_TYPE = 14
        MOVE_UNUSED_BUDGET = 15
        FORECASTING_CAMPAIGN_BUDGET = 16
        TARGET_ROAS_OPT_IN = 17
        RESPONSIVE_SEARCH_AD = 18
        MARGINAL_ROI_CAMPAIGN_BUDGET = 19
        USE_BROAD_MATCH_KEYWORD = 20
        RESPONSIVE_SEARCH_AD_ASSET = 21
        UPGRADE_SMART_SHOPPING_CAMPAIGN_TO_PERFORMANCE_MAX = 22
        RESPONSIVE_SEARCH_AD_IMPROVE_AD_STRENGTH = 23
        DISPLAY_EXPANSION_OPT_IN = 24
        UPGRADE_LOCAL_CAMPAIGN_TO_PERFORMANCE_MAX = 25
        RAISE_TARGET_CPA_BID_TOO_LOW = 26
        FORECASTING_SET_TARGET_ROAS = 27
        CALLOUT_ASSET = 28
        SITELINK_ASSET = 29
        CALL_ASSET = 30
        SHOPPING_ADD_AGE_GROUP = 31
        SHOPPING_ADD_COLOR = 32
        SHOPPING_ADD_GENDER = 33
        SHOPPING_ADD_GTIN = 34
        SHOPPING_ADD_MORE_IDENTIFIERS = 35
        SHOPPING_ADD_SIZE = 36
        SHOPPING_ADD_PRODUCTS_TO_CAMPAIGN = 37
        SHOPPING_FIX_DISAPPROVED_PRODUCTS = 38
        SHOPPING_TARGET_ALL_OFFERS = 39
        SHOPPING_FIX_SUSPENDED_MERCHANT_CENTER_ACCOUNT = 40
        SHOPPING_FIX_MERCHANT_CENTER_ACCOUNT_SUSPENSION_WARNING = 41
        SHOPPING_MIGRATE_REGULAR_SHOPPING_CAMPAIGN_OFFERS_TO_PERFORMANCE_MAX = (
            42
        )
        DYNAMIC_IMAGE_EXTENSION_OPT_IN = 43
        RAISE_TARGET_CPA = 44
        LOWER_TARGET_ROAS = 45
        PERFORMANCE_MAX_OPT_IN = 46
        IMPROVE_PERFORMANCE_MAX_AD_STRENGTH = 47
        MIGRATE_DYNAMIC_SEARCH_ADS_CAMPAIGN_TO_PERFORMANCE_MAX = 48
        FORECASTING_SET_TARGET_CPA = 49
        SET_TARGET_CPA = 50
        SET_TARGET_ROAS = 51
        MAXIMIZE_CONVERSION_VALUE_OPT_IN = 52
        IMPROVE_GOOGLE_TAG_COVERAGE = 53
        PERFORMANCE_MAX_FINAL_URL_OPT_IN = 54
        REFRESH_CUSTOMER_MATCH_LIST = 55
        CUSTOM_AUDIENCE_OPT_IN = 56
        LEAD_FORM_ASSET = 57
        IMPROVE_DEMAND_GEN_AD_STRENGTH = 58


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v19.enums.types import month_of_year


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.common",
    marshal="google.ads.googleads.v19",
    manifest={
        "DateRange",
        "YearMonthRange",
        "YearMonth",
    },
)


class DateRange(proto.Message):
    r"""A date range.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        start_date (str):
            The start date, in yyyy-mm-dd format. This
            date is inclusive.

            This field is a member of `oneof`_ ``_start_date``.
        end_date (str):
            The end date, in yyyy-mm-dd format. This date
            is inclusive.

            This field is a member of `oneof`_ ``_end_date``.
    """

    start_date: str = proto.Field(
        proto.STRING,
        number=3,
        optional=True,
    )
    end_date: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )


class YearMonthRange(proto.Message):
    r"""The year month range inclusive of the start and end months.
    Eg: A year month range to represent Jan 2020 would be: (Jan
    2020, Jan 2020).

    Attributes:
        start (google.ads.googleads.v19.common.types.YearMonth):
            The inclusive start year month.
        end (google.ads.googleads.v19.common.types.YearMonth):
            The inclusive end year month.
    """

    start: "YearMonth" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="YearMonth",
    )
    end: "YearMonth" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="YearMonth",
    )


class YearMonth(proto.Message):
    r"""Year month.

    Attributes:
        year (int):
            The year (for example, 2020).
        month (google.ads.googleads.v19.enums.types.MonthOfYearEnum.MonthOfYear):
            The month of the year. (for example,
            FEBRUARY).
    """

    year: int = proto.Field(
        proto.INT64,
        number=1,
    )
    month: month_of_year.MonthOfYearEnum.MonthOfYear = proto.Field(
        proto.ENUM,
        number=2,
        enum=month_of_year.MonthOfYearEnum.MonthOfYear,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "UserListStringRuleItemOperatorEnum",
    },
)


class UserListStringRuleItemOperatorEnum(proto.Message):
    r"""Supported rule operator for string type."""

    class UserListStringRuleItemOperator(proto.Enum):
        r"""Enum describing possible user list string rule item
        operators.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            CONTAINS (2):
                Contains.
            EQUALS (3):
                Equals.
            STARTS_WITH (4):
                Starts with.
            ENDS_WITH (5):
                Ends with.
            NOT_EQUALS (6):
                Not equals.
            NOT_CONTAINS (7):
                Not contains.
            NOT_STARTS_WITH (8):
                Not starts with.
            NOT_ENDS_WITH (9):
                Not ends with.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CONTAINS = 2
        EQUALS = 3
        STARTS_WITH = 4
        ENDS_WITH = 5
        NOT_EQUALS = 6
        NOT_CONTAINS = 7
        NOT_STARTS_WITH = 8
        NOT_ENDS_WITH = 9


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "PaymentsAccount",
    },
)


class PaymentsAccount(proto.Message):
    r"""A payments account, which can be used to set up billing for
    an Ads customer.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the payments account.
            PaymentsAccount resource names have the form:

            ``customers/{customer_id}/paymentsAccounts/{payments_account_id}``
        payments_account_id (str):
            Output only. A 16 digit ID used to identify a
            payments account.

            This field is a member of `oneof`_ ``_payments_account_id``.
        name (str):
            Output only. The name of the payments
            account.

            This field is a member of `oneof`_ ``_name``.
        currency_code (str):
            Output only. The currency code of the
            payments account. A subset of the currency codes
            derived from the ISO 4217 standard is supported.

            This field is a member of `oneof`_ ``_currency_code``.
        payments_profile_id (str):
            Output only. A 12 digit ID used to identify
            the payments profile associated with the
            payments account.

            This field is a member of `oneof`_ ``_payments_profile_id``.
        secondary_payments_profile_id (str):
            Output only. A secondary payments profile ID
            present in uncommon situations, for example,
            when a sequential liability agreement has been
            arranged.

            This field is a member of `oneof`_ ``_secondary_payments_profile_id``.
        paying_manager_customer (str):
            Output only. Paying manager of this payment
            account.

            This field is a member of `oneof`_ ``_paying_manager_customer``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    payments_account_id: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    currency_code: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    payments_profile_id: str = proto.Field(
        proto.STRING,
        number=11,
        optional=True,
    )
    secondary_payments_profile_id: str = proto.Field(
        proto.STRING,
        number=12,
        optional=True,
    )
    paying_manager_customer: str = proto.Field(
        proto.STRING,
        number=13,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "CriterionErrorEnum",
    },
)


class CriterionErrorEnum(proto.Message):
    r"""Container for enum describing possible criterion errors."""

    class CriterionError(proto.Enum):
        r"""Enum describing possible criterion errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CONCRETE_TYPE_REQUIRED (2):
                Concrete type of criterion is required for
                CREATE and UPDATE operations.
            INVALID_EXCLUDED_CATEGORY (3):
                The category requested for exclusion is
                invalid.
            INVALID_KEYWORD_TEXT (4):
                Invalid keyword criteria text.
            KEYWORD_TEXT_TOO_LONG (5):
                Keyword text should be less than 80 chars.
            KEYWORD_HAS_TOO_MANY_WORDS (6):
                Keyword text has too many words.
            KEYWORD_HAS_INVALID_CHARS (7):
                Keyword text has invalid characters or
                symbols.
            INVALID_PLACEMENT_URL (8):
                Invalid placement URL.
            INVALID_USER_LIST (9):
                Invalid user list criterion.
            INVALID_USER_INTEREST (10):
                Invalid user interest criterion.
            INVALID_FORMAT_FOR_PLACEMENT_URL (11):
                Placement URL has wrong format.
            PLACEMENT_URL_IS_TOO_LONG (12):
                Placement URL is too long.
            PLACEMENT_URL_HAS_ILLEGAL_CHAR (13):
                Indicates the URL contains an illegal
                character.
            PLACEMENT_URL_HAS_MULTIPLE_SITES_IN_LINE (14):
                Indicates the URL contains multiple comma
                separated URLs.
            PLACEMENT_IS_NOT_AVAILABLE_FOR_TARGETING_OR_EXCLUSION (15):
                Indicates the domain is blocked.
            INVALID_TOPIC_PATH (16):
                Invalid topic path.
            INVALID_YOUTUBE_CHANNEL_ID (17):
                The YouTube Channel Id is invalid.
            INVALID_YOUTUBE_VIDEO_ID (18):
                The YouTube Video Id is invalid.
            YOUTUBE_VERTICAL_CHANNEL_DEPRECATED (19):
                Indicates the placement is a YouTube vertical
                channel, which is no longer supported.
            YOUTUBE_DEMOGRAPHIC_CHANNEL_DEPRECATED (20):
                Indicates the placement is a YouTube
                demographic channel, which is no longer
                supported.
            YOUTUBE_URL_UNSUPPORTED (21):
                YouTube urls are not supported in Placement
                criterion. Use YouTubeChannel and YouTubeVideo
                criterion instead.
            CANNOT_EXCLUDE_CRITERIA_TYPE (22):
                Criteria type can not be excluded by the
                customer, like AOL account type cannot target
                site type criteria.
            CANNOT_ADD_CRITERIA_TYPE (23):
                Criteria type can not be targeted.
            CANNOT_EXCLUDE_SIMILAR_USER_LIST (26):
                Not allowed to exclude similar user list.
            CANNOT_ADD_CLOSED_USER_LIST (27):
                Not allowed to target a closed user list.
            CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SEARCH_ONLY_CAMPAIGNS (28):
                Not allowed to add display only UserLists to
                search only campaigns.
            CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SEARCH_CAMPAIGNS (29):
                Not allowed to add display only UserLists to
                search plus campaigns.
            CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SHOPPING_CAMPAIGNS (30):
                Not allowed to add display only UserLists to
                shopping campaigns.
            CANNOT_ADD_USER_INTERESTS_TO_SEARCH_CAMPAIGNS (31):
                Not allowed to add User interests to search
                only campaigns.
            CANNOT_SET_BIDS_ON_CRITERION_TYPE_IN_SEARCH_CAMPAIGNS (32):
                Not allowed to set bids for this criterion
                type in search campaigns
            CANNOT_ADD_URLS_TO_CRITERION_TYPE_FOR_CAMPAIGN_TYPE (33):
                Final URLs, URL Templates and
                CustomParameters cannot be set for the criterion
                types of Gender, AgeRange, UserList, Placement,
                MobileApp, and MobileAppCategory in search
                campaigns and shopping campaigns.
            INVALID_COMBINED_AUDIENCE (122):
                Invalid combined audience criterion.
            INVALID_CUSTOM_AFFINITY (96):
                Invalid custom affinity criterion.
            INVALID_CUSTOM_INTENT (97):
                Invalid custom intent criterion.
            INVALID_CUSTOM_AUDIENCE (121):
                Invalid custom audience criterion.
            INVALID_IP_ADDRESS (34):
                IP address is not valid.
            INVALID_IP_FORMAT (35):
                IP format is not valid.
            INVALID_MOBILE_APP (36):
                Mobile application is not valid.
            INVALID_MOBILE_APP_CATEGORY (37):
                Mobile application category is not valid.
            INVALID_CRITERION_ID (38):
                The CriterionId does not exist or is of the
                incorrect type.
            CANNOT_TARGET_CRITERION (39):
                The Criterion is not allowed to be targeted.
            CANNOT_TARGET_OBSOLETE_CRITERION (40):
                The criterion is not allowed to be targeted
                as it is deprecated.
            CRITERION_ID_AND_TYPE_MISMATCH (41):
                The CriterionId is not valid for the type.
            INVALID_PROXIMITY_RADIUS (42):
                Distance for the radius for the proximity
                criterion is invalid.
            INVALID_PROXIMITY_RADIUS_UNITS (43):
                Units for the distance for the radius for the
                proximity criterion is invalid.
            INVALID_STREETADDRESS_LENGTH (44):
                Street address in the address is not valid.
            INVALID_CITYNAME_LENGTH (45):
                City name in the address is not valid.
            INVALID_REGIONCODE_LENGTH (46):
                Region code in the address is not valid.
            INVALID_REGIONNAME_LENGTH (47):
                Region name in the address is not valid.
            INVALID_POSTALCODE_LENGTH (48):
                Postal code in the address is not valid.
            INVALID_COUNTRY_CODE (49):
                Country code in the address is not valid.
            INVALID_LATITUDE (50):
                Latitude for the GeoPoint is not valid.
            INVALID_LONGITUDE (51):
                Longitude for the GeoPoint is not valid.
            PROXIMITY_GEOPOINT_AND_ADDRESS_BOTH_CANNOT_BE_NULL (52):
                The Proximity input is not valid. Both
                address and geoPoint cannot be null.
            INVALID_PROXIMITY_ADDRESS (53):
                The Proximity address cannot be geocoded to a
                valid lat/long.
            INVALID_USER_DOMAIN_NAME (54):
                User domain name is not valid.
            CRITERION_PARAMETER_TOO_LONG (55):
                Length of serialized criterion parameter
                exceeded size limit.
            AD_SCHEDULE_TIME_INTERVALS_OVERLAP (56):
                Time interval in the AdSchedule overlaps with
                another AdSchedule.
            AD_SCHEDULE_INTERVAL_CANNOT_SPAN_MULTIPLE_DAYS (57):
                AdSchedule time interval cannot span multiple
                days.
            AD_SCHEDULE_INVALID_TIME_INTERVAL (58):
                AdSchedule time interval specified is
                invalid, endTime cannot be earlier than
                startTime.
            AD_SCHEDULE_EXCEEDED_INTERVALS_PER_DAY_LIMIT (59):
                The number of AdSchedule entries in a day
                exceeds the limit.
            AD_SCHEDULE_CRITERION_ID_MISMATCHING_FIELDS (60):
                CriteriaId does not match the interval of the
                AdSchedule specified.
            CANNOT_BID_MODIFY_CRITERION_TYPE (61):
                Cannot set bid modifier for this criterion
                type.
            CANNOT_BID_MODIFY_CRITERION_CAMPAIGN_OPTED_OUT (62):
                Cannot bid modify criterion, since it is
                opted out of the campaign.
            CANNOT_BID_MODIFY_NEGATIVE_CRITERION (63):
                Cannot set bid modifier for a negative
                criterion.
            BID_MODIFIER_ALREADY_EXISTS (64):
                Bid Modifier already exists. Use SET
                operation to update.
            FEED_ID_NOT_ALLOWED (65):
                Feed Id is not allowed in these Location
                Groups.
            ACCOUNT_INELIGIBLE_FOR_CRITERIA_TYPE (66):
                The account may not use the requested
                criteria type. For example, some accounts are
                restricted to keywords only.
            CRITERIA_TYPE_INVALID_FOR_BIDDING_STRATEGY (67):
                The requested criteria type cannot be used
                with campaign or ad group bidding strategy.
            CANNOT_EXCLUDE_CRITERION (68):
                The Criterion is not allowed to be excluded.
            CANNOT_REMOVE_CRITERION (69):
                The criterion is not allowed to be removed.
                For example, we cannot remove any of the device
                criterion.
            INVALID_PRODUCT_BIDDING_CATEGORY (76):
                Bidding categories do not form a valid path
                in the Shopping bidding category taxonomy.
            MISSING_SHOPPING_SETTING (77):
                ShoppingSetting must be added to the campaign
                before ProductScope criteria can be added.
            INVALID_MATCHING_FUNCTION (78):
                Matching function is invalid.
            LOCATION_FILTER_NOT_ALLOWED (79):
                Filter parameters not allowed for location
                groups targeting.
            INVALID_FEED_FOR_LOCATION_FILTER (98):
                Feed not found, or the feed is not an enabled
                location feed.
            LOCATION_FILTER_INVALID (80):
                Given location filter parameter is invalid
                for location groups targeting.
            CANNOT_SET_GEO_TARGET_CONSTANTS_WITH_FEED_ITEM_SETS (123):
                Cannot set geo target constants and feed item
                sets at the same time.
            CANNOT_SET_BOTH_ASSET_SET_AND_FEED (140):
                Cannot set both assetset and feed at the same
                time.
            CANNOT_SET_FEED_OR_FEED_ITEM_SETS_FOR_CUSTOMER (142):
                Cannot set feed or feed item sets for
                Customer.
            CANNOT_SET_ASSET_SET_FIELD_FOR_CUSTOMER (150):
                Cannot set AssetSet criteria for customer.
            CANNOT_SET_GEO_TARGET_CONSTANTS_WITH_ASSET_SETS (143):
                Cannot set geo target constants and asset
                sets at the same time.
            CANNOT_SET_ASSET_SETS_WITH_FEED_ITEM_SETS (144):
                Cannot set asset sets and feed item sets at
                the same time.
            INVALID_LOCATION_GROUP_ASSET_SET (141):
                The location group asset set id is invalid
            INVALID_LOCATION_GROUP_RADIUS (124):
                The location group radius is in the range but
                not at the valid increment.
            INVALID_LOCATION_GROUP_RADIUS_UNIT (125):
                The location group radius unit is invalid.
            CANNOT_ATTACH_CRITERIA_AT_CAMPAIGN_AND_ADGROUP (81):
                Criteria type cannot be associated with a
                campaign and its ad group(s) simultaneously.
            HOTEL_LENGTH_OF_STAY_OVERLAPS_WITH_EXISTING_CRITERION (82):
                Range represented by hotel length of stay's
                min nights and max nights overlaps with an
                existing criterion.
            HOTEL_ADVANCE_BOOKING_WINDOW_OVERLAPS_WITH_EXISTING_CRITERION (83):
                Range represented by hotel advance booking
                window's min days and max days overlaps with an
                existing criterion.
            FIELD_INCOMPATIBLE_WITH_NEGATIVE_TARGETING (84):
                The field is not allowed to be set when the
                negative field is set to true, for example, we
                don't allow bids in negative ad group or
                campaign criteria.
            INVALID_WEBPAGE_CONDITION (85):
                The combination of operand and operator in
                webpage condition is invalid.
            INVALID_WEBPAGE_CONDITION_URL (86):
                The URL of webpage condition is invalid.
            WEBPAGE_CONDITION_URL_CANNOT_BE_EMPTY (87):
                The URL of webpage condition cannot be empty
                or contain white space.
            WEBPAGE_CONDITION_URL_UNSUPPORTED_PROTOCOL (88):
                The URL of webpage condition contains an
                unsupported protocol.
            WEBPAGE_CONDITION_URL_CANNOT_BE_IP_ADDRESS (89):
                The URL of webpage condition cannot be an IP
                address.
            WEBPAGE_CONDITION_URL_DOMAIN_NOT_CONSISTENT_WITH_CAMPAIGN_SETTING (90):
                The domain of the URL is not consistent with
                the domain in campaign setting.
            WEBPAGE_CONDITION_URL_CANNOT_BE_PUBLIC_SUFFIX (91):
                The URL of webpage condition cannot be a
                public suffix itself.
            WEBPAGE_CONDITION_URL_INVALID_PUBLIC_SUFFIX (92):
                The URL of webpage condition has an invalid
                public suffix.
            WEBPAGE_CONDITION_URL_VALUE_TRACK_VALUE_NOT_SUPPORTED (93):
                Value track parameter is not supported in
                webpage condition URL.
            WEBPAGE_CRITERION_URL_EQUALS_CAN_HAVE_ONLY_ONE_CONDITION (94):
                Only one URL-EQUALS webpage condition is
                allowed in a webpage criterion and it cannot be
                combined with other conditions.
            WEBPAGE_CRITERION_NOT_SUPPORTED_ON_NON_DSA_AD_GROUP (95):
                A webpage criterion cannot be added to a
                non-DSA ad group.
            CANNOT_TARGET_USER_LIST_FOR_SMART_DISPLAY_CAMPAIGNS (99):
                Cannot add positive user list criteria in
                Smart Display campaigns.
            CANNOT_TARGET_PLACEMENTS_FOR_SEARCH_CAMPAIGNS (126):
                Cannot add positive placement criterion types
                in search campaigns.
            LISTING_SCOPE_TOO_MANY_DIMENSION_TYPES (100):
                Listing scope contains too many dimension
                types.
            LISTING_SCOPE_TOO_MANY_IN_OPERATORS (101):
                Listing scope has too many IN operators.
            LISTING_SCOPE_IN_OPERATOR_NOT_SUPPORTED (102):
                Listing scope contains IN operator on an
                unsupported dimension type.
            DUPLICATE_LISTING_DIMENSION_TYPE (103):
                There are dimensions with duplicate dimension
                type.
            DUPLICATE_LISTING_DIMENSION_VALUE (104):
                There are dimensions with duplicate dimension
                value.
            CANNOT_SET_BIDS_ON_LISTING_GROUP_SUBDIVISION (105):
                Listing group SUBDIVISION nodes cannot have
                bids.
            LISTING_GROUP_ERROR_IN_ANOTHER_OPERATION (169):
                Product group operation is invalid because
                another operation targeting the same AdGroupId
                is failing.
            INVALID_LISTING_GROUP_HIERARCHY (106):
                Ad group is invalid due to the listing groups
                it contains.
            LISTING_GROUP_TREE_WAS_INVALID_BEFORE_MUTATION (170):
                Tree was invalid before the mutation.
            LISTING_GROUP_UNIT_CANNOT_HAVE_CHILDREN (107):
                Listing group unit cannot have children.
            LISTING_GROUP_SUBDIVISION_REQUIRES_OTHERS_CASE (108):
                Subdivided listing groups must have an
                "others" case.
            LISTING_GROUP_REQUIRES_SAME_DIMENSION_TYPE_AS_SIBLINGS (109):
                Dimension type of listing group must be the
                same as that of its siblings.
            LISTING_GROUP_ALREADY_EXISTS (110):
                Listing group cannot be added to the ad group
                because it already exists.
            LISTING_GROUP_DOES_NOT_EXIST (111):
                Listing group referenced in the operation was
                not found in the ad group.
            LISTING_GROUP_CANNOT_BE_REMOVED (112):
                Recursive removal failed because listing
                group subdivision is being created or modified
                in this request.
            INVALID_LISTING_GROUP_TYPE (113):
                Listing group type is not allowed for
                specified ad group criterion type.
            LISTING_GROUP_ADD_MAY_ONLY_USE_TEMP_ID (114):
                Listing group in an ADD operation specifies a
                non temporary criterion id.
            LISTING_SCOPE_TOO_LONG (115):
                The combined length of dimension values of
                the Listing scope criterion is too long.
            LISTING_SCOPE_TOO_MANY_DIMENSIONS (116):
                Listing scope contains too many dimensions.
            LISTING_GROUP_TOO_LONG (117):
                The combined length of dimension values of
                the Listing group criterion is too long.
            LISTING_GROUP_TREE_TOO_DEEP (118):
                Listing group tree is too deep.
            INVALID_LISTING_DIMENSION (119):
                Listing dimension is invalid (for example,
                dimension contains illegal value, dimension type
                is represented with wrong class, etc). Listing
                dimension value can not contain "==" or "&+".
            INVALID_LISTING_DIMENSION_TYPE (120):
                Listing dimension type is either invalid for campaigns of
                this type or cannot be used in the current context.
                BIDDING_CATEGORY_Lx and PRODUCT_TYPE_Lx dimensions must be
                used in ascending order of their levels: L1, L2, L3, L4,
                L5... The levels must be specified sequentially and start
                from L1. Furthermore, an "others" Listing group cannot be
                subdivided with a dimension of the same type but of a higher
                level ("others" BIDDING_CATEGORY_L3 can be subdivided with
                BRAND but not with BIDDING_CATEGORY_L4).
            ADVERTISER_NOT_ON_ALLOWLIST_FOR_COMBINED_AUDIENCE_ON_DISPLAY (127):
                Customer is not on allowlist for composite
                audience in display campaigns.
            CANNOT_TARGET_REMOVED_COMBINED_AUDIENCE (128):
                Cannot target on a removed combined audience.
            INVALID_COMBINED_AUDIENCE_ID (129):
                Combined audience ID is invalid.
            CANNOT_TARGET_REMOVED_CUSTOM_AUDIENCE (130):
                Can not target removed combined audience.
            HOTEL_CHECK_IN_DATE_RANGE_OVERLAPS_WITH_EXISTING_CRITERION (131):
                Range represented by hotel check-in date's
                start date and end date overlaps with an
                existing criterion.
            HOTEL_CHECK_IN_DATE_RANGE_START_DATE_TOO_EARLY (132):
                Start date is earlier than earliest allowed
                value of yesterday UTC.
            HOTEL_CHECK_IN_DATE_RANGE_END_DATE_TOO_LATE (133):
                End date later is than latest allowed day of
                330 days in the future UTC.
            HOTEL_CHECK_IN_DATE_RANGE_REVERSED (134):
                Start date is after end date.
            BROAD_MATCH_MODIFIER_KEYWORD_NOT_ALLOWED (135):
                Broad match modifier (BMM) keywords can no
                longer be created. See
                https://ads-developers.googleblog.com/2021/06/broad-match-modifier-upcoming-changes.html.
            ONE_AUDIENCE_ALLOWED_PER_ASSET_GROUP (136):
                Only one audience is allowed in an asset
                group.
            AUDIENCE_NOT_ELIGIBLE_FOR_CAMPAIGN_TYPE (137):
                Audience is not supported for the specified
                campaign type.
            AUDIENCE_NOT_ALLOWED_TO_ATTACH_WHEN_AUDIENCE_GROUPED_SET_TO_FALSE (138):
                Audience is not allowed to attach when use_audience_grouped
                bit is set to false.
            CANNOT_TARGET_CUSTOMER_MATCH_USER_LIST (139):
                Targeting is not allowed for Customer Match
                lists as per Customer Match policy. See
                https://support.google.com/google-ads/answer/6299717.
            NEGATIVE_KEYWORD_SHARED_SET_DOES_NOT_EXIST (145):
                Cannot create a negative keyword list
                criterion with a shared set that does not exist.
            CANNOT_ADD_REMOVED_NEGATIVE_KEYWORD_SHARED_SET (146):
                Cannot create a negative keyword list with
                deleted shared set.
            CANNOT_HAVE_MULTIPLE_NEGATIVE_KEYWORD_LIST_PER_ACCOUNT (147):
                Can only have one Negative Keyword List per
                account.
            CUSTOMER_CANNOT_ADD_CRITERION_OF_THIS_TYPE (149):
                Only allowlisted customers can add criteria
                of this type.
            CANNOT_TARGET_SIMILAR_USER_LIST (151):
                Targeting for Similar audiences is not
                supported, since this feature has been
                deprecated. See
                https://support.google.com/google-ads/answer/12463119
                to learn more.
            CANNOT_ADD_AUDIENCE_SEGMENT_CRITERION_WHEN_AUDIENCE_GROUPED_IS_SET (152):
                Audience segment criteria cannot be added when
                use_audience_grouped bit is set.
            ONE_AUDIENCE_ALLOWED_PER_AD_GROUP (153):
                Only one audience is allowed in an ad group.
            INVALID_DETAILED_DEMOGRAPHIC (154):
                Invalid detailed demographics criterion.
            CANNOT_RECOGNIZE_BRAND (155):
                The brand criteria has a brand input that is
                not recognized as a valid brand.
            BRAND_SHARED_SET_DOES_NOT_EXIST (156):
                The brand_list.shared_set_id references a shared set that
                does not exist.
            CANNOT_ADD_REMOVED_BRAND_SHARED_SET (157):
                Cannot create a brand list with deleted
                shared set.
            ONLY_EXCLUSION_BRAND_LIST_ALLOWED_FOR_CAMPAIGN_TYPE (158):
                Brand list can only be negatively targeted
                for the campaign type.
            LOCATION_TARGETING_NOT_ELIGIBLE_FOR_RESTRICTED_CAMPAIGN (166):
                Cannot positively target locations outside of
                restricted area for campaign.
            ONLY_INCLUSION_BRAND_LIST_ALLOWED_FOR_AD_GROUPS (171):
                Ad group level brand list criteria only
                support inclusionary targeting. Negative
                targeting at this level is not supported.
            CANNOT_ADD_REMOVED_PLACEMENT_LIST_SHARED_SET (172):
                Cannot create a placement list with deleted
                shared set.
            PLACEMENT_LIST_SHARED_SET_DOES_NOT_EXIST (173):
                The placement_list.shared_set_id references a shared set
                that does not exist.
            AI_MAX_MUST_BE_ENABLED (174):
                This feature is only available for AI Max
                campaigns.
            NOT_AVAILABLE_FOR_AI_MAX_CAMPAIGNS (175):
                This feature is not available for AI Max
                campaigns.
            MISSING_EU_POLITICAL_ADVERTISING_SELF_DECLARATION (176):
                The operation failed because the campaign is
                missing the self-declaration on political
                advertising status in the EU.
            INVALID_CAMPAIGN_TYPE_FOR_THIRD_PARTY_PARTNER_DATA_LIST (177):
                Targeting this UserList is not allowed for
                this campaign type.
            CANNOT_ADD_USER_LIST_PENDING_PRIVACY_REVIEW (178):
                The user list cannot be used while it is
                pending privacy review.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CONCRETE_TYPE_REQUIRED = 2
        INVALID_EXCLUDED_CATEGORY = 3
        INVALID_KEYWORD_TEXT = 4
        KEYWORD_TEXT_TOO_LONG = 5
        KEYWORD_HAS_TOO_MANY_WORDS = 6
        KEYWORD_HAS_INVALID_CHARS = 7
        INVALID_PLACEMENT_URL = 8
        INVALID_USER_LIST = 9
        INVALID_USER_INTEREST = 10
        INVALID_FORMAT_FOR_PLACEMENT_URL = 11
        PLACEMENT_URL_IS_TOO_LONG = 12
        PLACEMENT_URL_HAS_ILLEGAL_CHAR = 13
        PLACEMENT_URL_HAS_MULTIPLE_SITES_IN_LINE = 14
        PLACEMENT_IS_NOT_AVAILABLE_FOR_TARGETING_OR_EXCLUSION = 15
        INVALID_TOPIC_PATH = 16
        INVALID_YOUTUBE_CHANNEL_ID = 17
        INVALID_YOUTUBE_VIDEO_ID = 18
        YOUTUBE_VERTICAL_CHANNEL_DEPRECATED = 19
        YOUTUBE_DEMOGRAPHIC_CHANNEL_DEPRECATED = 20
        YOUTUBE_URL_UNSUPPORTED = 21
        CANNOT_EXCLUDE_CRITERIA_TYPE = 22
        CANNOT_ADD_CRITERIA_TYPE = 23
        CANNOT_EXCLUDE_SIMILAR_USER_LIST = 26
        CANNOT_ADD_CLOSED_USER_LIST = 27
        CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SEARCH_ONLY_CAMPAIGNS = 28
        CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SEARCH_CAMPAIGNS = 29
        CANNOT_ADD_DISPLAY_ONLY_LISTS_TO_SHOPPING_CAMPAIGNS = 30
        CANNOT_ADD_USER_INTERESTS_TO_SEARCH_CAMPAIGNS = 31
        CANNOT_SET_BIDS_ON_CRITERION_TYPE_IN_SEARCH_CAMPAIGNS = 32
        CANNOT_ADD_URLS_TO_CRITERION_TYPE_FOR_CAMPAIGN_TYPE = 33
        INVALID_COMBINED_AUDIENCE = 122
        INVALID_CUSTOM_AFFINITY = 96
        INVALID_CUSTOM_INTENT = 97
        INVALID_CUSTOM_AUDIENCE = 121
        INVALID_IP_ADDRESS = 34
        INVALID_IP_FORMAT = 35
        INVALID_MOBILE_APP = 36
        INVALID_MOBILE_APP_CATEGORY = 37
        INVALID_CRITERION_ID = 38
        CANNOT_TARGET_CRITERION = 39
        CANNOT_TARGET_OBSOLETE_CRITERION = 40
        CRITERION_ID_AND_TYPE_MISMATCH = 41
        INVALID_PROXIMITY_RADIUS = 42
        INVALID_PROXIMITY_RADIUS_UNITS = 43
        INVALID_STREETADDRESS_LENGTH = 44
        INVALID_CITYNAME_LENGTH = 45
        INVALID_REGIONCODE_LENGTH = 46
        INVALID_REGIONNAME_LENGTH = 47
        INVALID_POSTALCODE_LENGTH = 48
        INVALID_COUNTRY_CODE = 49
        INVALID_LATITUDE = 50
        INVALID_LONGITUDE = 51
        PROXIMITY_GEOPOINT_AND_ADDRESS_BOTH_CANNOT_BE_NULL = 52
        INVALID_PROXIMITY_ADDRESS = 53
        INVALID_USER_DOMAIN_NAME = 54
        CRITERION_PARAMETER_TOO_LONG = 55
        AD_SCHEDULE_TIME_INTERVALS_OVERLAP = 56
        AD_SCHEDULE_INTERVAL_CANNOT_SPAN_MULTIPLE_DAYS = 57
        AD_SCHEDULE_INVALID_TIME_INTERVAL = 58
        AD_SCHEDULE_EXCEEDED_INTERVALS_PER_DAY_LIMIT = 59
        AD_SCHEDULE_CRITERION_ID_MISMATCHING_FIELDS = 60
        CANNOT_BID_MODIFY_CRITERION_TYPE = 61
        CANNOT_BID_MODIFY_CRITERION_CAMPAIGN_OPTED_OUT = 62
        CANNOT_BID_MODIFY_NEGATIVE_CRITERION = 63
        BID_MODIFIER_ALREADY_EXISTS = 64
        FEED_ID_NOT_ALLOWED = 65
        ACCOUNT_INELIGIBLE_FOR_CRITERIA_TYPE = 66
        CRITERIA_TYPE_INVALID_FOR_BIDDING_STRATEGY = 67
        CANNOT_EXCLUDE_CRITERION = 68
        CANNOT_REMOVE_CRITERION = 69
        INVALID_PRODUCT_BIDDING_CATEGORY = 76
        MISSING_SHOPPING_SETTING = 77
        INVALID_MATCHING_FUNCTION = 78
        LOCATION_FILTER_NOT_ALLOWED = 79
        INVALID_FEED_FOR_LOCATION_FILTER = 98
        LOCATION_FILTER_INVALID = 80
        CANNOT_SET_GEO_TARGET_CONSTANTS_WITH_FEED_ITEM_SETS = 123
        CANNOT_SET_BOTH_ASSET_SET_AND_FEED = 140
        CANNOT_SET_FEED_OR_FEED_ITEM_SETS_FOR_CUSTOMER = 142
        CANNOT_SET_ASSET_SET_FIELD_FOR_CUSTOMER = 150
        CANNOT_SET_GEO_TARGET_CONSTANTS_WITH_ASSET_SETS = 143
        CANNOT_SET_ASSET_SETS_WITH_FEED_ITEM_SETS = 144
        INVALID_LOCATION_GROUP_ASSET_SET = 141
        INVALID_LOCATION_GROUP_RADIUS = 124
        INVALID_LOCATION_GROUP_RADIUS_UNIT = 125
        CANNOT_ATTACH_CRITERIA_AT_CAMPAIGN_AND_ADGROUP = 81
        HOTEL_LENGTH_OF_STAY_OVERLAPS_WITH_EXISTING_CRITERION = 82
        HOTEL_ADVANCE_BOOKING_WINDOW_OVERLAPS_WITH_EXISTING_CRITERION = 83
        FIELD_INCOMPATIBLE_WITH_NEGATIVE_TARGETING = 84
        INVALID_WEBPAGE_CONDITION = 85
        INVALID_WEBPAGE_CONDITION_URL = 86
        WEBPAGE_CONDITION_URL_CANNOT_BE_EMPTY = 87
        WEBPAGE_CONDITION_URL_UNSUPPORTED_PROTOCOL = 88
        WEBPAGE_CONDITION_URL_CANNOT_BE_IP_ADDRESS = 89
        WEBPAGE_CONDITION_URL_DOMAIN_NOT_CONSISTENT_WITH_CAMPAIGN_SETTING = 90
        WEBPAGE_CONDITION_URL_CANNOT_BE_PUBLIC_SUFFIX = 91
        WEBPAGE_CONDITION_URL_INVALID_PUBLIC_SUFFIX = 92
        WEBPAGE_CONDITION_URL_VALUE_TRACK_VALUE_NOT_SUPPORTED = 93
        WEBPAGE_CRITERION_URL_EQUALS_CAN_HAVE_ONLY_ONE_CONDITION = 94
        WEBPAGE_CRITERION_NOT_SUPPORTED_ON_NON_DSA_AD_GROUP = 95
        CANNOT_TARGET_USER_LIST_FOR_SMART_DISPLAY_CAMPAIGNS = 99
        CANNOT_TARGET_PLACEMENTS_FOR_SEARCH_CAMPAIGNS = 126
        LISTING_SCOPE_TOO_MANY_DIMENSION_TYPES = 100
        LISTING_SCOPE_TOO_MANY_IN_OPERATORS = 101
        LISTING_SCOPE_IN_OPERATOR_NOT_SUPPORTED = 102
        DUPLICATE_LISTING_DIMENSION_TYPE = 103
        DUPLICATE_LISTING_DIMENSION_VALUE = 104
        CANNOT_SET_BIDS_ON_LISTING_GROUP_SUBDIVISION = 105
        LISTING_GROUP_ERROR_IN_ANOTHER_OPERATION = 169
        INVALID_LISTING_GROUP_HIERARCHY = 106
        LISTING_GROUP_TREE_WAS_INVALID_BEFORE_MUTATION = 170
        LISTING_GROUP_UNIT_CANNOT_HAVE_CHILDREN = 107
        LISTING_GROUP_SUBDIVISION_REQUIRES_OTHERS_CASE = 108
        LISTING_GROUP_REQUIRES_SAME_DIMENSION_TYPE_AS_SIBLINGS = 109
        LISTING_GROUP_ALREADY_EXISTS = 110
        LISTING_GROUP_DOES_NOT_EXIST = 111
        LISTING_GROUP_CANNOT_BE_REMOVED = 112
        INVALID_LISTING_GROUP_TYPE = 113
        LISTING_GROUP_ADD_MAY_ONLY_USE_TEMP_ID = 114
        LISTING_SCOPE_TOO_LONG = 115
        LISTING_SCOPE_TOO_MANY_DIMENSIONS = 116
        LISTING_GROUP_TOO_LONG = 117
        LISTING_GROUP_TREE_TOO_DEEP = 118
        INVALID_LISTING_DIMENSION = 119
        INVALID_LISTING_DIMENSION_TYPE = 120
        ADVERTISER_NOT_ON_ALLOWLIST_FOR_COMBINED_AUDIENCE_ON_DISPLAY = 127
        CANNOT_TARGET_REMOVED_COMBINED_AUDIENCE = 128
        INVALID_COMBINED_AUDIENCE_ID = 129
        CANNOT_TARGET_REMOVED_CUSTOM_AUDIENCE = 130
        HOTEL_CHECK_IN_DATE_RANGE_OVERLAPS_WITH_EXISTING_CRITERION = 131
        HOTEL_CHECK_IN_DATE_RANGE_START_DATE_TOO_EARLY = 132
        HOTEL_CHECK_IN_DATE_RANGE_END_DATE_TOO_LATE = 133
        HOTEL_CHECK_IN_DATE_RANGE_REVERSED = 134
        BROAD_MATCH_MODIFIER_KEYWORD_NOT_ALLOWED = 135
        ONE_AUDIENCE_ALLOWED_PER_ASSET_GROUP = 136
        AUDIENCE_NOT_ELIGIBLE_FOR_CAMPAIGN_TYPE = 137
        AUDIENCE_NOT_ALLOWED_TO_ATTACH_WHEN_AUDIENCE_GROUPED_SET_TO_FALSE = 138
        CANNOT_TARGET_CUSTOMER_MATCH_USER_LIST = 139
        NEGATIVE_KEYWORD_SHARED_SET_DOES_NOT_EXIST = 145
        CANNOT_ADD_REMOVED_NEGATIVE_KEYWORD_SHARED_SET = 146
        CANNOT_HAVE_MULTIPLE_NEGATIVE_KEYWORD_LIST_PER_ACCOUNT = 147
        CUSTOMER_CANNOT_ADD_CRITERION_OF_THIS_TYPE = 149
        CANNOT_TARGET_SIMILAR_USER_LIST = 151
        CANNOT_ADD_AUDIENCE_SEGMENT_CRITERION_WHEN_AUDIENCE_GROUPED_IS_SET = 152
        ONE_AUDIENCE_ALLOWED_PER_AD_GROUP = 153
        INVALID_DETAILED_DEMOGRAPHIC = 154
        CANNOT_RECOGNIZE_BRAND = 155
        BRAND_SHARED_SET_DOES_NOT_EXIST = 156
        CANNOT_ADD_REMOVED_BRAND_SHARED_SET = 157
        ONLY_EXCLUSION_BRAND_LIST_ALLOWED_FOR_CAMPAIGN_TYPE = 158
        LOCATION_TARGETING_NOT_ELIGIBLE_FOR_RESTRICTED_CAMPAIGN = 166
        ONLY_INCLUSION_BRAND_LIST_ALLOWED_FOR_AD_GROUPS = 171
        CANNOT_ADD_REMOVED_PLACEMENT_LIST_SHARED_SET = 172
        PLACEMENT_LIST_SHARED_SET_DOES_NOT_EXIST = 173
        AI_MAX_MUST_BE_ENABLED = 174
        NOT_AVAILABLE_FOR_AI_MAX_CAMPAIGNS = 175
        MISSING_EU_POLITICAL_ADVERTISING_SELF_DECLARATION = 176
        INVALID_CAMPAIGN_TYPE_FOR_THIRD_PARTY_PARTNER_DATA_LIST = 177
        CANNOT_ADD_USER_LIST_PENDING_PRIVACY_REVIEW = 178


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.common.types import policy
from google.ads.googleads.v20.enums.types import policy_approval_status
from google.ads.googleads.v20.enums.types import policy_review_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.common",
    marshal="google.ads.googleads.v20",
    manifest={
        "PolicySummary",
    },
)


class PolicySummary(proto.Message):
    r"""Contains policy summary information.

    Attributes:
        policy_topic_entries (MutableSequence[google.ads.googleads.v20.common.types.PolicyTopicEntry]):
            The list of policy findings.
        review_status (google.ads.googleads.v20.enums.types.PolicyReviewStatusEnum.PolicyReviewStatus):
            Where in the review process the resource is.
        approval_status (google.ads.googleads.v20.enums.types.PolicyApprovalStatusEnum.PolicyApprovalStatus):
            The overall approval status, which is
            calculated based on the status of its individual
            policy topic entries.
    """

    policy_topic_entries: MutableSequence[policy.PolicyTopicEntry] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=policy.PolicyTopicEntry,
        )
    )
    review_status: (
        policy_review_status.PolicyReviewStatusEnum.PolicyReviewStatus
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=policy_review_status.PolicyReviewStatusEnum.PolicyReviewStatus,
    )
    approval_status: (
        policy_approval_status.PolicyApprovalStatusEnum.PolicyApprovalStatus
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=policy_approval_status.PolicyApprovalStatusEnum.PolicyApprovalStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.errors",
    marshal="google.ads.googleads.v19",
    manifest={
        "BatchJobErrorEnum",
    },
)


class BatchJobErrorEnum(proto.Message):
    r"""Container for enum describing possible batch job errors."""

    class BatchJobError(proto.Enum):
        r"""Enum describing possible request errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CANNOT_MODIFY_JOB_AFTER_JOB_STARTS_RUNNING (2):
                The batch job cannot add more operations or
                run after it has started running.
            EMPTY_OPERATIONS (3):
                The operations for an AddBatchJobOperations
                request were empty.
            INVALID_SEQUENCE_TOKEN (4):
                The sequence token for an
                AddBatchJobOperations request was invalid.
            RESULTS_NOT_READY (5):
                Batch job results can only be retrieved once
                the job is finished.
            INVALID_PAGE_SIZE (6):
                The page size for ListBatchJobResults was
                invalid.
            CAN_ONLY_REMOVE_PENDING_JOB (7):
                The batch job cannot be removed because it
                has started running.
            CANNOT_LIST_RESULTS (8):
                The batch job cannot be listed due to
                unexpected errors such as duplicate checkpoints.
            ASSET_GROUP_AND_ASSET_GROUP_ASSET_TRANSACTION_FAILURE (9):
                The request contains interdependent
                AssetGroup and AssetGroupAsset operations that
                are treated atomically as a single transaction,
                and one or more of the operations in that
                transaction failed, which caused the entire
                transaction, and therefore this mutate
                operation, to fail. The operations that caused
                the transaction to fail can be found in the
                consecutive AssetGroup or AssetGroupAsset
                results with the same asset group id. The mutate
                operation will be successful once the remaining
                errors in the transaction are fixed.
            ASSET_GROUP_LISTING_GROUP_FILTER_TRANSACTION_FAILURE (10):
                The request contains interdependent
                AssetGroupListingGroupFilter operations that are
                treated atomically as a single transaction, and
                one or more of the operations in that
                transaction failed, which caused the entire
                transaction, and therefore this mutate
                operation, to fail. The operations that caused
                the transaction to fail can be found in the
                consecutive AssetGroupListingGroupFilter results
                with the same asset group id. The mutate
                operation will be successful once the remaining
                errors in the transaction are fixed.
            REQUEST_TOO_LARGE (11):
                The AddBatchJobOperationsRequest is too
                large. Split the request into smaller requests.
                The maximum allowed request size is 10484504
                bytes.
            CAMPAIGN_AND_CAMPAIGN_ASSET_TRANSACTION_FAILURE (12):
                This error indicates a failed transaction
                involving interdependent Campaign and
                CampaignAsset operations that are treated
                atomically as a single transaction. Because some
                operations within the transaction failed, the
                entire set of changes was rejected. Related
                error details are found in the results for the
                Campaign and CampaignAssets sharing the same
                Campaign ID. The transaction will succeed after
                all associated errors are resolved.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CANNOT_MODIFY_JOB_AFTER_JOB_STARTS_RUNNING = 2
        EMPTY_OPERATIONS = 3
        INVALID_SEQUENCE_TOKEN = 4
        RESULTS_NOT_READY = 5
        INVALID_PAGE_SIZE = 6
        CAN_ONLY_REMOVE_PENDING_JOB = 7
        CANNOT_LIST_RESULTS = 8
        ASSET_GROUP_AND_ASSET_GROUP_ASSET_TRANSACTION_FAILURE = 9
        ASSET_GROUP_LISTING_GROUP_FILTER_TRANSACTION_FAILURE = 10
        REQUEST_TOO_LARGE = 11
        CAMPAIGN_AND_CAMPAIGN_ASSET_TRANSACTION_FAILURE = 12


__all__ = tuple(sorted(__protobuf__.manifest))
